@@ -8,20 +8,23 @@
  */
 
 
-#include"config.h"
-#include"types.h"
-#include"lib.h"
-#include"interrupt.h"
-#include"proc.h"
-#include"time.h"
-#include"errno.h"
-#include"mm.h"
-#include"mp.h"
-#include"sp.h"
-#include"pci.h"
-#include"fs.h"
-#include"device.h"
-#include<test.h>
+#include <config.h>
+#include <types.h>
+#include <lib.h>
+#include <interrupt.h>
+#include <proc.h>
+#include <time.h>
+#include <errno.h>
+#include <mm.h>
+#include <mp.h>
+#include <sp.h>
+#include <pci.h>
+#include <fs.h>
+#include <device.h>
+#include <console.h>
+#include <sys/poll.h>
+#include <debug.h>
+#include <test.h>
 
 
 enum{
@@ -41,8 +44,8 @@ enum{
 	CHK_BIT=0x1,
 
 	/* IRQ number */
-	PRIM_IRQ=14,
-	SECOND_IRQ=15,
+	PRIM_IRQ	= 14,
+	SECOND_IRQ	= 15,
 
 	/* Transfer mode bit in identify infomation */
 	PIO4=0x2,		/* PIO mode4 */
@@ -58,12 +61,15 @@ enum{
 	U_DMA1=0x2,		/* Ultra DMA mode1 */
 	U_DMA0=0x1,		/* Ultra DMA mode0 */
 
-	/* Conect device infomation */
-	ATA=1,
-	ATAPI=2,
-	PIO=1,		/* PIO transfer mode */
-	M_DMA=2,	/* Multi DMA transfer mode */
-	U_DMA=3,	/* Ultra DMA transfer mode */
+	/* ǥХ */
+	NODEV = 0,
+	ATA,
+	ATAPI,
+
+	/* ž */
+	PIO = 1,	/* PIO transfer mode */
+	M_DMA,		/* Multi DMA transfer mode */
+	U_DMA,		/* Ultra DMA transfer mode */
 
 	/* Interrupt mode */
 	INTR_DISABLE=0,
@@ -156,7 +162,7 @@ typedef struct{
 
 /* Conect device. */
 typedef struct{
-	int type;				/* ATA=1 or ATAPI=2 */
+	int type;				/* ATA or ATAPI */
 	int mode;				/* PIO=0,Multi DMA=1,Ultra DMA=2 */
 	int sectorSize;			/* Secter size */
 	uint allSectors;		/* LBA all sectors */
@@ -176,8 +182,8 @@ static CONECT_DEV conectDev[2][2]={			/* Conect device infomation */
 	{{0,0,0,0,0,NULL},{0,0,0,0,0,NULL}},
 	{{0,0,0,0,0,NULL},{0,0,0,0,0,NULL}}
 };
-static WAIT_INTR wait_intr_queue[2];			/* Ԥ */
-static uchar irqNum[2]={PRIM_IRQ,SECOND_IRQ};	/* IRQ number */
+static WAIT_INTR waitIntr[2];						// Ԥѹ¤
+static uchar irqNum[2] = {PRIM_IRQ, SECOND_IRQ};	// IRQֹ
 
 
 /************************************************************************************************
@@ -407,22 +413,22 @@ static int writePio(int host,int dev,void *buf,int block,int count)
  */
 static int readDma(int host,int dev,void *buf,int sector_size,int count)
 {
-	/* Set PRD */
+	// Set PRD
 	prd[host].physAddr = buf;
 	prd[host].count = sector_size * count | PRD_EOT;
 	outdw(ide_base[host] + IDE_BMIDTP,(uint)&prd[host]);
 
-	/* Mask interrupt. */
-/*	set_irq_mask(irqNum[host]);*/
+	// Mask interrupt.
+//	set_irq_mask(irqNum[host]);
 
 	/*
 	 * Хޥơ쥸γߥե饰򥯥ꥢʤ
 	 * ߤȯʤ褦
 	 */
-	outb(ide_base[host] + IDE_BMIS,0x6);			/* Clear interrupt bit and error bit */
-	outb(ide_base[host] + IDE_BMIC,0x9);			/* Start read Bus Master */
-	wait_intr(&wait_intr_queue[host],IDE_BM_TIMEOUT,TASK_DEVICE_WAIT);		/* Wait interrupt */
-	outb(ide_base[host] + IDE_BMIC,0);				/* Stop Bus Master */
+	outb(ide_base[host] + IDE_BMIS,0x6);							// Clear interrupt bit and error bit.
+	outb(ide_base[host] + IDE_BMIC,0x9);							// Start read Bus Master.
+	wait_intr(&waitIntr[host], IDE_BM_TIMEOUT, TASK_DEVICE_WAIT);	// Wait interrupt.
+	outb(ide_base[host] + IDE_BMIC,0);								// Stop Bus Master.
 
 	return inb(reg[host].str);
 }
@@ -436,19 +442,19 @@ static int readDma(int host,int dev,void *buf,int sector_size,int count)
  */
 static int writeDma(int host,int dev,void *buf,int sector_size,int count)
 {
-	/* Set PRD */
-	prd[host].physAddr=buf;
-	prd[host].count=sector_size*count|PRD_EOT;
-	outdw(ide_base[host]+IDE_BMIDTP,(uint)&prd[host]);
+	// Set PRD
+	prd[host].physAddr = buf;
+	prd[host].count = sector_size * count | PRD_EOT;
+	outdw(ide_base[host] + IDE_BMIDTP, (uint)&prd[host]);
 
 	/*
 	 * Хޥơ쥸γߥե饰򥯥ꥢʤ
 	 * ߤȯʤ褦
 	 */
-	outb(ide_base[host]+IDE_BMIS,0x6);			/* Clear interrupt bit and error bit */
-	outb(ide_base[host]+IDE_BMIC,0x1);			/* Start write Bus Master */
-	wait_intr(&wait_intr_queue[host],IDE_BM_TIMEOUT,TASK_DEVICE_WAIT);		/* Wait interrupt */
-	outb(ide_base[host]+IDE_BMIC,0);			/* Stop Bus Master */
+	outb(ide_base[host] + IDE_BMIS, 0x6);							// Clear interrupt bit and error bit.
+	outb(ide_base[host] + IDE_BMIC, 0x1);							// Start write Bus Master.
+	wait_intr(&waitIntr[host], IDE_BM_TIMEOUT, TASK_DEVICE_WAIT);	// Wait interrupt.
+	outb(ide_base[host] + IDE_BMIC, 0);								// Stop Bus Master.
 
 	return inb(reg[host].str);
 }
@@ -801,19 +807,21 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 		for(i=0;i<6;++i)outw(dtr,((short*)param->packet)[i]);
 
 		/* Overrapped */
-		if(param->feutures&PACK_OVL)
+		if (param->feutures & PACK_OVL)
 		{
-			wait_intr(&wait_intr_queue[host],2000,TASK_DEVICE_WAIT);			/* Wait interrupt */
+			wait_intr(&waitIntr[host],2000,TASK_DEVICE_WAIT);	/* Wait interrupt */
+			error = (int)inb(reg[host].str);
 
-			error=(int)inb(reg[host].str);
 			if(error&ERR_BIT)
 				return -EDERRE;		/* Packet command Error */
 			if((error&DRQ_BIT)==0)
-				return 0;				/* Non data transfer */
+				return 0;			/* Non data transfer */
 
-			if((error=deviceSelect(host,dev,0))!=0)return error;
-			wait_intr(&wait_intr_queue[host],2000,TASK_DEVICE_WAIT);		/* Wait interrupt */
-			outb(reg[host].cmr,0xa2);					/* Issue service command */
+			if((error=deviceSelect(host,dev,0))!=0)
+				return error;
+			wait_intr(&waitIntr[host],2000,TASK_DEVICE_WAIT);	/* Wait interrupt */
+
+			outb(reg[host].cmr,0xa2);		/* Issue service command */
 			if((checkBusy(reg[host].str)&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)
 				return -EDBUSY;
 		}
@@ -844,8 +852,10 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 		/* Overrapped */
 		if(param->feutures&PACK_OVL)
 		{
-			if((error=deviceSelect(host,dev,0))!=0)return error;
-			wait_intr(&wait_intr_queue[host],2000,TASK_DEVICE_WAIT);			/* Wait interrupt */
+			if((error = deviceSelect(host,dev,0)) != 0)
+				return error;
+			wait_intr(&waitIntr[host],2000,TASK_DEVICE_WAIT);			/* Wait interrupt */
+			
 			outb(reg[host].cmr,0xa2);						/* Issue service command */
 			if((checkBusy(reg[host].str)&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)
 				return -EDBUSY;
@@ -1329,9 +1339,9 @@ static int primIntrHandler()
 /***************************************
 	printk("IRQ14\n");
 ***************************************/
-	wake_intr(&wait_intr_queue[0],TASK_DEVICE_WAIT);
+	return wake_intr(&waitIntr[0],TASK_DEVICE_WAIT);
 
-	return 1;
+//	return 1;
 }
 
 
@@ -1345,9 +1355,7 @@ static int secondIntrHandler()
 /***************************************
 	printk("IRQ15\n");
 ***************************************/
-	wake_intr(&wait_intr_queue[1],TASK_DEVICE_WAIT);
-
-	return 1;
+	return wake_intr(&waitIntr[1],TASK_DEVICE_WAIT);
 }
 
 
@@ -1398,10 +1406,9 @@ static inline int transfer(int host,int dev,int mode,void *buf,size_t blocks,siz
 
 
 /*
- * GLOBAL
  * File operation interface
  */
-static int openHda()
+static int openHda(dev_t dev, int oflags, int devtype, d_thread_t *proc)
 {
 	switch(conectDev[0][0].type)
 	{
@@ -1411,7 +1418,7 @@ static int openHda()
 	}
 }
 
-static int openHdb()
+static int openHdb(dev_t dev, int oflags, int devtype, d_thread_t *proc)
 {
 	switch(conectDev[0][1].type)
 	{
@@ -1421,7 +1428,7 @@ static int openHdb()
 	}
 }
 
-static int openHdc()
+static int openHdc(dev_t dev, int oflags, int devtype, d_thread_t *proc)
 {
 	switch(conectDev[1][0].type)
 	{
@@ -1431,7 +1438,7 @@ static int openHdc()
 	}
 }
 
-static int openHdd()
+static int openHdd(dev_t dev, int oflags, int devtype, d_thread_t *proc)
 {
 	switch(conectDev[1][1].type)
 	{
@@ -1441,77 +1448,82 @@ static int openHdd()
 	}
 }
 
-static int readHda(void *buf,size_t size,size_t begin)
-{
-	return transfer(0,0,READ,buf,size,begin);
-}
-
-static int readHdb(void *buf,size_t size,size_t begin)
-{
-	return transfer(0,1,READ,buf,size,begin);
-}
-
-static int readHdc(void *buf,size_t size,size_t begin)
-{
-	return transfer(1,0,READ,buf,size,begin);
-}
-
-static int readHdd(void *buf,size_t size,size_t begin)
-{
-	return transfer(1,1,READ,buf,size,begin);
-}
-
-static int writeHda(void *buf,size_t size,size_t begin)
-{
-	return transfer(0,0,WRITE,buf,size,begin);
-}
-
-static int writeHdb(void *buf,size_t size,size_t begin)
-{
-	return transfer(0,1,WRITE,buf,size,begin);
-}
-
-static int writeHdc(void *buf,size_t size,size_t begin)
-{
-	return transfer(1,0,WRITE,buf,size,begin);
-}
-
-static int writeHdd(void *buf,size_t size,size_t begin)
-{
-	return transfer(1,1,WRITE,buf,size,begin);
-}
-
-static int ioctlHda(int cmd,...)
+static int close(dev_t dev, int fflag, int devtype, d_thread_t *proc)
 {
 	return 0;
 }
 
-static int ioctlHdb(int cmd,...)
+static int readHda(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(0, 0, READ, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int readHdb(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(0, 1, READ, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int readHdc(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(1, 0, READ, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int readHdd(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(1, 1, READ, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int writeHda(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(0, 0, WRITE, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int writeHdb(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(0, 1, WRITE, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int writeHdc(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(1, 0, WRITE, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int writeHdd(dev_t dev, struct uio *uio, int ioflag)
+{
+	return transfer(1, 1, WRITE, uio->uio_iov->iov_base, uio->uio_iov->iov_len, uio->uio_offset);
+}
+
+static int ioctlHda(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 {
 	return 0;
 }
 
-static int ioctlHdc(int cmd,...)
+static int ioctlHdb(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 {
 	return 0;
 }
 
-static int ioctlHdd(int cmd,...)
+static int ioctlHdc(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 {
 	return 0;
 }
 
-static int select(int flag,int polling)
+static int ioctlHdd(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 {
-	return 1;
+	return 0;
+}
+
+static int poll(dev_t dev, int events, d_thread_t *p)
+{
+	return POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM;
 }
 
 
-static DEV_INFO hd_info[2][2]={
-	{{"hda",0,0,0,openHda,readHda,writeHda,ioctlHda,select},
-	 {"hdb",0,0,0,openHdb,readHdb,writeHdb,ioctlHdb,select}},
-	{{"hdc",0,0,0,openHdc,readHdc,writeHdc,ioctlHdc,select},
-	 {"hdd",0,0,0,openHdd,readHdd,writeHdd,ioctlHdd,select}}
+static struct cdevsw hd_info[2][2]={
+	{{openHda, close, readHda, writeHda, ioctlHda, poll, NULL, NULL, "hda", 0, NULL,  NULL, D_DISK,},
+	 {openHdb, close, readHdb, writeHdb, ioctlHdb, poll, NULL, NULL, "hdb", 0, NULL,  NULL, D_DISK,}},
+	{{openHdc, close, readHdc, writeHdc, ioctlHdc, poll, NULL, NULL, "hdc", 0, NULL,  NULL, D_DISK,},
+	 {openHdd, close, readHdd, writeHdd, ioctlHdd, poll, NULL, NULL, "hdd", 0, NULL,  NULL, D_DISK,}}
 };
 
 
@@ -1525,18 +1537,22 @@ static int testAtapi(int host,int dev)
 {
 	int error;
 
-
-	for(;testUnitReady(host,dev)!=0;)
-	{
-		error=requestSense(host,dev);
-		if(error==0)break;
-		else if(error<0)return error;
+	for(; testUnitReady(host, dev) != 0;){
+		error = requestSense(host, dev);
+		if (error == 0){
+			break;
+		}
+		else if (error < 0){
+			return error;
+		}
 	}
 
 	/* Read sector size and max sector number */
-	if((error=readCapacity(host,dev))!=0)return error;
-	hd_info[host][dev].last_blk=conectDev[host][dev].allSectors-1;
-	hd_info[host][dev].sector_size=conectDev[host][dev].sectorSize;
+	if ((error = readCapacity(host, dev)) != 0){
+		return error;
+	}
+	hd_info[host][dev].last_blk = conectDev[host][dev].allSectors - 1;
+	hd_info[host][dev].sector_size = conectDev[host][dev].sectorSize;
 
 	return 0;
 }
@@ -1629,7 +1645,7 @@ int initAta()
 
 				/* Print device infomation */
 				convertBigToLittle(id_info->model,40);
-				printk("%s : %s, %s\n",hd_info[i][j].name,id_info->model,"ATA DISK drive");
+				printk("%s : %s, %s\n", hd_info[i][j].d_name, id_info->model, "ATA DISK drive");
 
 				/* LBA all sectors */
 				conectDev[i][j].allSectors=(uint)id_info->lba_all_sect[1]<<16|(uint)id_info->lba_all_sect[0];
@@ -1665,7 +1681,7 @@ int initAta()
 						media="ATAPI OTHER drive";
 						break;
 				}
-				printk("%s : %s, %s\n",hd_info[i][j].name,id_info->model,media);
+				printk("%s : %s, %s\n", hd_info[i][j].d_name, id_info->model, media);
 
 				conectDev[i][j].flag=id_info->iordy;
 				conectDev[i][j].type=ATAPI;
@@ -1677,12 +1693,11 @@ int initAta()
 			idleImmediateDevice(i,j);
 
 			/* Set DMA or PIO mode. */
-			if(changeMode(i,j,M_DMA)!=0)
-				if((error=changeMode(i,j,PIO))!=0)
-					printk("Transfer mode set error : %x\n",error);
-
-			/* Regster to dev filesystem */
-			regist_device(&hd_info[i][j]);
+			if (changeMode(i, j, M_DMA) != 0){
+				if ((error = changeMode(i, j, PIO)) != 0){
+					printk("Transfer mode set error : %x\n", error);
+				}
+			}
 		}
 	}
 
@@ -1692,17 +1707,20 @@ int initAta()
 	 * ⤦ٳǧ
 	 * ǥХˤäƤϡƱۥȤ¸ߤʤӥ֤ΤޤޤˤʤäƤޤ
 	 */
-	for(i=0;i<2;++i)
-		for(j=0;j<2;++j)
-		{
-			outb(reg[i].dhr,j<<4);
+	for (i = 0; i < 2; ++i){
+		for (j = 0; j < 2; ++j){
+			outb(reg[i].dhr, j << 4);
 			mili_timer(5);
-			if((inb(reg[i].str)&BSY_BIT)==BSY_BIT)
-			{
-				if((error=softReset(i))!=0)return error;				/* soft reset */
-				for(k=0;k<2;++k)changeMode(i,k,conectDev[i][k].mode);	/* set transfer mode */
+			if ((inb(reg[i].str) & BSY_BIT) == BSY_BIT){
+				if ((error = softReset(i)) != 0){	/* soft reset */
+					return error;
+				}
+				for (k = 0; k < 2; ++k){
+					changeMode(i, k, conectDev[i][k].mode);	/* set transfer mode */
+				}
 			}
 		}
+	}
 
 	/*
 	 * ߤ
@@ -1713,9 +1731,22 @@ int initAta()
 	release_irq_mask(SECOND_IRQ);
 	mili_timer(2);
 
-	irq_entry[IRQ14]=primIntrHandler;
-	irq_entry[IRQ15]=secondIntrHandler;
+	irq_entry[IRQ14] = primIntrHandler;
+	irq_entry[IRQ15] = secondIntrHandler;
 
+	/* ǥХե륷ƥϿ */
+	for (i = 0; i < 2; ++i){
+		for (j = 0; j < 2; ++j){
+			if (conectDev[i][j].type != NODEV){
+				make_dev(&hd_info[i][j], 0, 0, 0, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, "ata");
+			}
+			if (conectDev[i][j].type == ATA){
+				/* ѡƥǥϿ */
+				registBlockPartition(hd_info[i][j].d_name);
+			}
+		}
+	}
+	
 	return 0;
 }
 
