@@ -10,7 +10,8 @@
 #define console_h
 
 
-#include"types.h"
+#include<types.h>
+#include<proc.h>
 
 
 #ifndef ASM_FILE
@@ -23,13 +24,16 @@ enum{
 };
 
 
-extern void releaseTerm(PROC *);
-extern void initKeyboard();
 extern int registConsole();
 extern int initConsole();
+extern void initKeyboard();
 extern int writeConsole(void*,size_t,size_t);
 extern int cnsoleSetTopProc();
 
+#ifdef DEBUG
+extern void displayTop(int, const char*, int);
+#endif
+
 
 #endif
 
