@@ -14,7 +14,7 @@
 
 
 #include<lib.h>
-#include<share/errno.h>
+#include<sys/errno.h>
 
 
 #ifdef DEBUG
