@@ -12,13 +12,14 @@
 #define interrupt_h
 
 
-#include<types.h>
-#include<proc.h>
-#include<share/syscall.h>
-#include<share/segment.h>
+#include <types.h>
+#include <proc.h>
+#include <signal.h>
+#include <sys/syscall.h>
+#include <share/segment.h>
 
 
-/* IRQȥ꡼ơ֥ */
+// IRQȥ꡼ơ֥
 #define IRQ0			0
 #define IRQ1			1
 #define IRQ3			3
@@ -37,55 +38,26 @@
 #define FLASH_PAGEDIR	16
 #define APIC_TIMER		17
 
-
-#define FPU_SAVE_SIZE 108	/* FPU֥Хȥ */
-
+// FPU֥Хȥ
+#define FPU_SAVE_SIZE 108
 
 #ifndef ASM_FILE
 
 
 enum{
-	IRQ_ENTRY=APIC_TIMER+1,	/* IRQϥɥ饨ȥ꡼ */
+	IRQ_ENTRY=APIC_TIMER+1,	// IRQϥɥ饨ȥ꡼
 
-	/* IDT flag */
-	IDT_INTR=0xe00,		/* Intrrupt gate */
-	IDT_TRAP=0xf00,		/* Trap gate */
+	// IDT flag.
+	IDT_INTR=0xe00,	// Intrrupt gate
+	IDT_TRAP=0xf00,	// Trap gate
 };
 
 
 typedef int (*SYSCALL)();
 
 
-/*
- * Interrupt stack frame
- */
+// 㳰ߥåե졼
 typedef struct{
-	char fpu[FPU_SAVE_SIZE];
-/*	uint gs;
-	uint fs;*/
-	uint ds;
-	uint es;
-	uint edi;
-	uint esi;
-	uint ebp;
-	uint esp;
-	uint ebx;
-	uint edx;
-	uint ecx;
-	uint eax;
-	uint eip;
-	uint cs;
-	uint eflags;
-	uint user_esp;
-	uint user_ss;
-}TRAP_FRAME;
-
-
-/*
- * Exception stack frame
- */
-typedef struct{
-	char fpu[FPU_SAVE_SIZE];
 /*	uint gs;
 	uint fs;*/
 	uint ds;
@@ -104,38 +76,9 @@ typedef struct{
 	uint eflags;
 	uint user_esp;
 	uint user_ss;
-}EXCEPT_FRAME;
+}INTERRUPT_FRAME;
 
-/*
- * temp except frame.
- */
-typedef struct{
-	int except;
-	char fpu[FPU_SAVE_SIZE];
-	uint error;
-	uint eip;
-	uint cs;
-	uint eflags;
-	uint user_esp;
-	uint user_ss;
-}TMP_EXCEPT_FRAME;
-
-/*
- * debug frame.
- */
-typedef struct{
-	char fpu[FPU_SAVE_SIZE];
-	uint eip;
-	uint cs;
-	uint eflags;
-	uint user_esp;
-	uint user_ss;
-}DEBUG_FRAME;
-
-
-/*
- * System call frame.
- */
+// ƥॳ륹åե졼
 typedef struct{
 	uint es;
 	uint ds;
@@ -255,18 +198,15 @@ extern int sys_sleep();
 extern void wait_task();
 extern void returnExit(void*);
 extern uint setContext(uint);
-
 extern int (*irq_entry[])();
 extern void (*irq_eoi)(uchar);
 extern void (*release_irq_mask)(uchar);
 extern void (*set_irq_mask)(uchar);
-
 extern int init_syscall();
-
 extern void set_idt(int,void*,int);
 extern void set_gate(int,void*,uchar);
-extern void doUserHandler(int,void(*)(int),uint,uint*,uint*);
-extern void returnUserHandler();
+extern void callUserHandler(uint pageEnt, uint *pageTbl, SIG_ACTION userEip, uint userEsp, uint fpuSave, uint *o_backEsp);
+extern void returnCallUserHandler(uint kernelEsp, uint pageEnt, uint *pageTbl);
 
 
 #endif
