@@ -12,35 +12,55 @@
 #define lib_h
 
 
-#include"types.h"
-#include"segment.h"
+#include <types.h>
+#include <segment.h>
+#include <sys/uio.h>
 
 
 #ifndef ASM_FILE
 
 
-#define ROUNDUP(a,b) (((uint)(a)+(b)-1)/(b)*(b))
-#define ROUNDDOWN(a,b) ((uint)(a)/(b)*(b))
-#define ROUNDUP_DIV(a,b) (((uint)(a)+(b)-1)/(b))
+enum{
+	BYTE_BIT=8,		/* 1ХȤΥӥåȿ */
+};
 
 
+#define ROUNDUP(a, b) (((uint)(a) + (b) - 1) / (b) * (b))
+#define ROUNDDOWN(a, b) ((uint)(a) / (b) * (b))
+#define ROUNDUP_DIV(a, b) (((uint)(a) + (b) - 1) / (b))
+#define OFFSETOF(type, member) ((size_t) &((type*)0)->member)
+#define	MAX(a, b)		(((a) < (b))? (b) : (a))
+#define	MIN(a, b)		(((a) < (b))? (a) : (b))
+#define ARRAY_LENGTH(x)	(sizeof(x) / sizeof(x[0]))
+#define SWAP(a, b)	{int tmp = b; b = a; a = tmp;}
+#define WAIT_LOOP(condition)	{LOOP:	if (condition){goto LOOP;}}
+
 extern int (*write_tty)(void*,size_t,size_t);
 extern void printk(const char*,...);
 extern int sprintk(char *,const char*,...);
 extern void idle();
 extern int read_tr();
 extern uint64 rdtsc();
+extern void fpuSave(uint);
+extern void fpuStore(uint);
+extern void clearTs();
+extern void setTs();
+extern uint getFualtAddr();
 extern void micro_timer(uint);
 extern void mili_timer(uint);
 extern char *strcpy(char*,const char*);
 extern int strcmp(const char*,const char*);
-extern size_t strlen(const char*);
 extern int atoi(const char*);
 extern void *memcpy(void*,const void*, size_t);
 extern void memCpyBack(void*,const void*, size_t);
 extern void *memset(void*,int,size_t);
 extern void *memcpy_to_user(void *,const void*,size_t);
 extern void *memcpy_from_user(void *,const void*,size_t);
+extern int copyIovec(const char*,const int,struct iovec*,const int,int*,int*);
+
+#ifdef DEBUG
+extern void printDebug(int,const char*,...);
+#endif
 
 extern inline void outb(int port,uchar value)
 {
@@ -105,26 +125,6 @@ extern inline uint indw(int port)
 	return rest;
 }
 
-extern inline void rotateR(uint *value,int cnt)
-{
-	asm volatile(
-		"movl	(%%eax),%%edx\n"
-		"rorl	%%cl,%%edx\n"
-		"movl	%%edx,(%%eax)\n"
-		::"a"(value),"c"(cnt)
-	);
-}
-
-extern inline void rotateL(uint *value,int cnt)
-{
-	asm volatile(
-		"movl	(%%eax),%%edx\n"
-		"roll	%%cl,%%edx\n"
-		"movl	%%edx,(%%eax)\n"
-		::"a"(value),"c"(cnt)
-	);
-}
-
 extern inline void *memmove(void *s1,const void *s2,size_t n)
 {
 	if((s1>s2)&&((char*)s1<(char*)s2+n))memCpyBack(s1,s2,n);
@@ -155,30 +155,26 @@ extern inline char *strcpy(char *s1,const char *s2)
 	return s1;
 }
 
-extern inline char *strncpy(char *s1,const char *s2,size_t n)
-{
-	while(--n>=0)s1[n]=s2[n];
-
-	return s1;
-}
-
-
 extern inline size_t strlen(const char *s)
 {
-	const char *p=s;
+	const char *p = s;
 
 
-	while(*p++!='\0');
+	while(*p++ != '\0');
 
 	return (size_t)(p-s-1);
 }
 
-extern inline int strcmp(const char *s1,const char *s2)
+extern inline int strcmp(const char *s1, const char *s2)
 {
-	while(*s1==*s2++)
-		if(*s1++=='\0')return 0;
-
-	if((uchar)*s1>(uchar)*--s2)return 1;
+	while (*s1 == *s2++){
+		if (*s1++ == '\0'){
+			return 0;
+		}
+	}
+	if ((uchar)*s1 > (uchar)*--s2){
+		return 1;
+	}
 	return -1;
 }
 
