@@ -18,9 +18,6 @@
 #include<sys/system.h>
 
 
-#define PAGE_SIZE 0x1000	/* Page size */
-
-
 #ifndef ASM_FILE
 
 
@@ -34,31 +31,42 @@ enum{
 typedef struct{
 	uint *pageDir;
 	int pageDirGate;		/* ڡǥ쥯ȥåȡ */
-	uint lastReadPage;		/* Last read only page address. */
-	uint lastPageAddr;		/* ƺѤߥڡ饹ȥɥ쥹+1 */
+	uint lastPageAddr;		/* ƺѤߥڡ饹ȥɥ쥹 */
+	uint firstStackAddr;	/* ѤߥåƬɥ쥹 */
 }MM_STRUCT;
 
+/*
+ * ꡼¤
+ */
+typedef struct{
+	const int dummy;
+}MEM_ARRAY;
+
 
 extern void *kmalloc(size_t);
 extern void kfree(void*);
+extern MEM_ARRAY *allocArray(size_t);
+extern void freeArray(MEM_ARRAY *);
+extern MEM_ARRAY *reallocArray(MEM_ARRAY *,size_t);
+extern int writeArray(MEM_ARRAY *,const int,const size_t,const void *);
+extern void copyArray(MEM_ARRAY *,const int,const size_t,void *);
 extern int initBackupDev(const char*);
-extern void updateVm(void*);
+extern void updateVm();
 extern uint getLastLinearAddr();
-extern int checkMem(void*);
+extern int checkMem(const void*,size_t);
 extern void initMm();
 extern void *initPaging();
-extern void *forkPage(MM_STRUCT*,void**);
-extern int loadBinary(OPEN_F*,size_t,size_t,uint,int);
-extern uint setArgAndEnv(char**, char**);
-extern void releaseUserTextDataPage(void*);
-extern void *getPhysAddrFromLinearAddr(void*,int*);
+extern int forkPage(MM_STRUCT*, int, void**, void**);
+extern int loadBinary(EXEC_FD*,size_t,size_t,uint,int);
+extern uint setUserStack(char**, char**);
+extern void releaseUserPage();
 extern uint *getPageDir();
-/**********************************************************/
-extern uint *getKernelStackEntry();
+extern uint getPhysicalFromLogicalAddr(PROC*, uint);
 extern uint addUserStack(int);
 extern void delAddedUserStack();
-/**********************************************************/
 extern int sys_brk(void*);
+extern int doUserHandler(int,void(*)(int));
+extern void returnUserHandler();
 
 
 #endif
