@@ -8,19 +8,20 @@
  */
 
 
-#include"types.h"
-#include"config.h"
-#include"errno.h"
-#include"lib.h"
-#include"proc.h"
-#include"interrupt.h"
-#include"mm.h"
-#include"time.h"
-#include"lock.h"
-#include"time.h"
-#include"pci.h"
-#include"sp.h"
-#include"mp.h"
+#include<types.h>
+#include<config.h>
+#include<errno.h>
+#include<lib.h>
+#include<proc.h>
+#include<interrupt.h>
+#include<mm.h>
+#include<time.h>
+#include<lock.h>
+#include<time.h>
+#include<pci.h>
+#include<console.h>
+#include<sp.h>
+#include<mp.h>
 
 
 enum{
@@ -119,19 +120,24 @@ static uint task_count;										/* 1ư֤APICޡ */
  * smpƥΥ󥰤Ԥ
  * returns : Task switch on
  */
-static int apictimer_handler(TRAP_FRAME frame)
+static int apictimer_handler(INTERRUPT_FRAME frame)
 {
-/*******************************************/
+/*****************************************************************************/
 	{
-		static char value[MAX_CPU]={0,0,0,0};
-		char *p=(char*)(0xb8000+80);
-		int cpu;
+		static char value[MAX_CPU] = {'0','0','0','0','0','0','0','0'};
+		int cpu = get_current_cpu();
 
-		cpu=get_current_cpu();
-		*(p+cpu*4)=(value[cpu]%10)+'0';
+		if ('9' < value[cpu]){
+			value[cpu] = '0';
+		}
+		displayTop(60 + cpu, &value[cpu],1);
 		++value[cpu];
+		
+//		if (get_current_task() == (PROC*)0x150800){
+//			printDebug(80 * 1 + 70, "%x",frame.eip );
+//		}
 	}
-/********************************************/
+/****************************************************************************/
 	local_apic[LOCAL_APIC_ICNT]=task_count;	/* restart APIC TIMER */
 
 	return 1;
@@ -253,9 +259,9 @@ void issue_interrupt_to_cpu(int cpu,int vecter)
  * Page directory flash handler
  * returns : Task switch on
  */
-int flash_pagedir_handler(TRAP_FRAME frame)
+int flash_pagedir_handler(INTERRUPT_FRAME frame)
 {
-	updateVm(get_current_task()->mm_struct);
+	updateVm();
 	local_apic[LOCAL_APIC_EOI]=0;		/* issue EOI */
 
 	return 1;
@@ -457,9 +463,11 @@ int SetPciIrqInIoapic(PCI_INFO *pci,int vecter)
 	int pci_irq;
 
 
-	if((pci_irq=getPciIrq(pci))==-1)return -EDNOFUNC;
-	pci_irq+=IO_APIC_PCIIRQ;
-	write_ioapic(ioapic_lrt[pci_irq],read_ioapic(ioapic_lrt[pci_irq])|ioapic_intr_vect[vecter]);
+	if ((pci_irq = getPciIrq(pci)) == -1){
+		return -EDNOFUNC;
+	}
+	pci_irq += IO_APIC_PCIIRQ;
+	write_ioapic(ioapic_lrt[pci_irq],read_ioapic(ioapic_lrt[pci_irq]) | ioapic_intr_vect[vecter]);
 
 	return 0;
 }
@@ -475,9 +483,11 @@ void releasePciIrqIoapicMask(PCI_INFO *pci)
 	int pci_irq;
 
 
-	if((pci_irq=getPciIrq(pci))==-1)return;
-	pci_irq+=IO_APIC_PCIIRQ;
-	write_ioapic(ioapic_lrt[pci_irq],read_ioapic(ioapic_lrt[pci_irq])&~IOREDTBL_MASK);
+	if ((pci_irq = getPciIrq(pci)) == -1){
+		return;
+	}
+	pci_irq += IO_APIC_PCIIRQ;
+	write_ioapic(ioapic_lrt[pci_irq], read_ioapic(ioapic_lrt[pci_irq]) & ~IOREDTBL_MASK);
 }
 
 
@@ -542,7 +552,7 @@ void init_ioapic(int cpu)
 	write_ioapic(IO_APIC_HRT4 ,0xff000000);
 	write_ioapic(IO_APIC_LRT5 ,IOREDTBL_MASK|IOREDTBL_MODE_LOGIC|IOREDTBL_DELIV_LOWPRT|IRQ5_VECT);
 	write_ioapic(IO_APIC_HRT5 ,0xff000000);
-	write_ioapic(IO_APIC_LRT6 ,IOREDTBL_MASK|IOREDTBL_MODE_PHYS|IOREDTBL_DELIV_FIX|IRQ6_VECT);
+	write_ioapic(IO_APIC_LRT6 ,IOREDTBL_MASK|IOREDTBL_MODE_PHYS |IOREDTBL_DELIV_FIX   |IRQ6_VECT);
 	write_ioapic(IO_APIC_HRT6 ,0x00000000);
 	write_ioapic(IO_APIC_LRT7 ,IOREDTBL_MASK|IOREDTBL_MODE_LOGIC|IOREDTBL_DELIV_LOWPRT|IRQ7_VECT);
 	write_ioapic(IO_APIC_HRT7 ,0xff000000);
@@ -558,9 +568,9 @@ void init_ioapic(int cpu)
 	write_ioapic(IO_APIC_HRT12,0xff000000);
 	write_ioapic(IO_APIC_LRT13,IOREDTBL_MASK|IOREDTBL_MODE_LOGIC|IOREDTBL_DELIV_LOWPRT|IRQ13_VECT);
 	write_ioapic(IO_APIC_HRT13,0xff000000);
-	write_ioapic(IO_APIC_LRT14,IOREDTBL_MASK|IOREDTBL_MODE_PHYS|IOREDTBL_DELIV_FIX|IRQ14_VECT);
+	write_ioapic(IO_APIC_LRT14,IOREDTBL_MASK|IOREDTBL_MODE_PHYS |IOREDTBL_DELIV_FIX   |IRQ14_VECT);
 	write_ioapic(IO_APIC_HRT14,0x00000000);
-	write_ioapic(IO_APIC_LRT15,IOREDTBL_MASK|IOREDTBL_MODE_PHYS|IOREDTBL_DELIV_FIX|IRQ15_VECT);
+	write_ioapic(IO_APIC_LRT15,IOREDTBL_MASK|IOREDTBL_MODE_PHYS |IOREDTBL_DELIV_FIX   |IRQ15_VECT);
 	write_ioapic(IO_APIC_HRT15,0x00000000);
 	write_ioapic(IO_APIC_LRT16,IOREDTBL_MASK|IOREDTBL_MODE_LOGIC|IOREDTBL_DELIV_LOWPRT|IOREDTBL_POLLOW);
 	write_ioapic(IO_APIC_HRT16,0xff000000);
@@ -666,7 +676,7 @@ static void boot_acpu()
 	local_apic[LOCAL_APIC_ICR1]=value;
 	while(local_apic[LOCAL_APIC_ICR1]&0x1000);
 
-	/* INIT_IPI Deassert mode */
+	/* INIT_IPI DeASSERT mode */
 	value=local_apic[LOCAL_APIC_ICR1];
 	value=(value&0xfff32000)|0xc8500;
 	local_apic[LOCAL_APIC_ICR1]=value;
