@@ -29,7 +29,7 @@
 
 /* PUBLIC */
 ETHER_DEVICE ethDev[ETHER_DEVICE_MAX];
-static int etherNum=0;					/* Eathernet device number. */
+static int etherNum;					/* Eathernet device number. */
 
 
 /**************************************************************************
@@ -50,18 +50,20 @@ int searchDevnum(uint ip,uint *buf)
 {
 	int i;
 
-
-	for(i=0;i<etherNum;++i)
-		if((ip&ethDev[i].netmask)==ethDev[i].subnet)
-		{
-			*buf=ip;
+	for (i = 0; i < etherNum; ++i){
+		if ((ip & ethDev[i].netmask) == ethDev[i].subnet){
+			*buf = ip;
 			return i;
 		}
+	}
 
 	/* ǥեȥȥء */
-	*buf=IP_DEFAULT_GATEWAY;
-	for(i=0;i<etherNum;++i)
-		if((IP_DEFAULT_GATEWAY&ethDev[i].netmask)==ethDev[i].subnet)return i;
+	*buf = IP_DEFAULT_GATEWAY;
+	for (i = 0; i < etherNum; ++i){
+		if ((IP_DEFAULT_GATEWAY & ethDev[i].netmask) == ethDev[i].subnet){
+			return i;
+		}
+	}
 
 	return 0;
 }
@@ -127,37 +129,37 @@ printk("ETHER FRAME type=%x\n"
  **************************************************************************/
 
 
-/*
- * GLOBAL
- * Set ethernet driver infomation.
- * parameters : ethernet number,DEV_INFO address,mac address
- * return : 0 or error number.
- */
-int setEtherDevice(int ethnum,DEV_INFO *devinf,char *mac)
-{
-	if(ethnum>=ETHER_DEVICE_MAX)return -E2BIG;
+//================================== PUBLIC =============================================
 
-	ethDev[ethnum].dev=devinf;
-	memcpy(ethDev[ethnum].mac,mac,6);
+
+// Set ethernet driver infomation.
+// return : 0 or error number.
+int setEtherDevice(int ethnum, struct ifnet *ifnet, char *mac)
+{
+	if (ETHER_DEVICE_MAX <= ethnum){
+		return -E2BIG;
+	}
+
+	ethDev[ethnum].ifnet = ifnet;
+	memcpy(ethDev[ethnum].mac, mac, 6);
 
 	/*
 	 * IPɥ쥹ꡣ
 	 * Note!
 	 *  ѹͽꡣ
 	 */
-	switch(ethnum)
-	{
+	switch(ethnum){
 		case 0:
-			ethDev[0].ip=IP_ADDRESS0;
-			ethDev[0].netmask=IP_SUBNETMASK0;
-			ethDev[0].subnet=IP_ADDRESS0&IP_SUBNETMASK0;
-			ethDev[0].mtu=1500;
+			ethDev[0].ip = IP_ADDRESS0;
+			ethDev[0].netmask = IP_SUBNETMASK0;
+			ethDev[0].subnet = IP_ADDRESS0&IP_SUBNETMASK0;
+			ethDev[0].mtu = 1500;
 			break;
 		case 1:
-			ethDev[1].ip=IP_ADDRESS1;
-			ethDev[1].netmask=IP_SUBNETMASK1;
-			ethDev[1].subnet=IP_ADDRESS1&IP_SUBNETMASK1;
-			ethDev[1].mtu=1500;
+			ethDev[1].ip = IP_ADDRESS1;
+			ethDev[1].netmask = IP_SUBNETMASK1;
+			ethDev[1].subnet = IP_ADDRESS1&IP_SUBNETMASK1;
+			ethDev[1].mtu = 1500;
 			break;
 	}
 
