@@ -10,7 +10,7 @@
 #define ethernet_h
 
 
-#include<device.h>
+#include<net/if_var.h>
 
 
 #ifndef ASM_FILE
@@ -31,7 +31,7 @@ enum{
 
 /* Ethernet device infomation. */
 typedef struct{
-	DEV_INFO *dev;
+	struct ifnet *ifnet;
 	uint      ip;		/* IP address. */
 	uint      netmask;	/* ֥ͥåȥޥ */
 	uint      subnet;	/* ֥ͥåȡ */
@@ -50,10 +50,10 @@ typedef struct{
 extern ETHER_DEVICE ethDev[];
 
 
-extern int searchDevnum(uint,uint*);
+extern int searchDevnum(uint, uint*);
 extern uint32_t getSrcip(uint32_t);
-extern int receiveEther(int num,ETHER_FRAME *frame);
-extern int setEtherDevice(int ethnum,DEV_INFO *devinf,char *mac);
+extern int receiveEther(int num, ETHER_FRAME *frame);
+extern int setEtherDevice(int, struct ifnet *, char *);
 extern int initEther();
 
 
