@@ -11,14 +11,16 @@
 
 
 #include<types.h>
-#include<net/ip.h>
-#include<share/icmp.h>
+#include<net/net.h>
+#include<net/netlib.h>
+#include<sys/icmp.h>
 
 
 #ifndef ASM_FILE
 
 
 extern int receiveIcmp(IP_HEADER*);
+extern void releaseSockIcmp(SOCKET*);
 extern int initIcmp();
 
 
