@@ -12,8 +12,8 @@
 
 #include<types.h>
 #include<net/net.h>
-#include<share/in.h>
-#include<share/ip.h>
+#include<netinet/in.h>
+#include<sys/ip.h>
 
 
 #ifndef ASM_FILE
@@ -22,6 +22,7 @@
 enum{
 	/* IP header TOP value. */
 	IP_HEAD_TOS_ICMP=0x0,
+	IP_SIZE_MAX = 1500,
 };
 
 
@@ -35,8 +36,10 @@ typedef struct{
 }PSEUDO_HEADER;
 
 
-extern int transIp(TRANS_BUF_INFO*,uint,uchar,int);
+extern int transIp(SOCKET*, TRANS_BUF_INFO*, uint, uint16_t);
 extern int receiveIp(IP_HEADER*);
+extern void setIpTask(SOCKET*, IP_HEADER*);
+extern int initIp();
 
 
 #endif
