@@ -13,11 +13,16 @@
 #include<types.h>
 #include<net/net.h>
 #include<net/ip.h>
+#include<mm.h>
 
 
 #ifndef ASM_FILE
 
 
+enum{
+	TCP_MSS_SIZE = 1460,		/* ZOgőTCYB */
+};
+
 typedef struct{
 	uint16_t srcport;
 	uint16_t dstport;
@@ -32,8 +37,9 @@ typedef struct{
 }TCP_HEADER;
 
 
-extern void releaseSckFromConect(SOCKET*);
+extern int releaseSockFromConect(SOCKET*);
 extern int receiveTcp(IP_HEADER*);
+extern void doTcpService(SOCKET*, IP_HEADER*);
 extern int initTcp();
 
 
