@@ -11,7 +11,7 @@
 
 
 #include<types.h>
-#include<net/ip.h>
+#include<net/net.h>
 
 
 #ifndef ASM_FILE
@@ -27,6 +27,7 @@ typedef struct{
 
 
 extern int receiveUdp(IP_HEADER*);
+extern void doUdpService(SOCKET*, IP_HEADER*);
 extern int initUdp();
 extern void releaseSockUdp(SOCKET*);
 
