@@ -84,7 +84,6 @@ static char *uart_name[5]={
  */
 static int uart[2]={0,0};					/* UARTμ */
 static WAIT_INTR wait_intr_queue[2];		/* Ԥ */
-/*static WAIT_QUEUE wait_queue[2];*/
 static int fifo_buf_size[2]={1,1};			/* FIFO buffer size */
 
 
@@ -94,16 +93,6 @@ static void set_boudrate(int,ushort);
 static int write_ttys(int,const char*,size_t);
 static int write_com(int,char*,size_t);
 static int detect_uart(int);
-static int write_ttys0(void*,size_t,size_t);
-static int write_ttys1(void*,size_t,size_t);
-static int ioctl_ttys0(int,int,...);
-static int ioctl_ttys1(int,int,...);
-
-
-static DEV_INFO com_info[2]={
-	{"ttys0",0,0,0,NULL,NULL,write_ttys0,ioctl_ttys0},
-	{"ttys1",0,0,0,NULL,NULL,write_ttys1,ioctl_ttys1}
-};
 
 
 /*
@@ -112,22 +101,18 @@ static DEV_INFO com_info[2]={
  */
 int com1_intr_handler()
 {
-/***************************************/
-	printk("Interrupt IRQ4\n");
-/***************************************/
 	switch(inb(reg[0].iir)&0xf)
 	{
 		case TRANSMIT_EMPTY:
-			wake_intr(&wait_intr_queue[0],TASK_DEVICE_WAIT);
-			break;
+			return wake_intr(&wait_intr_queue[0],TASK_DEVICE_WAIT);
 		case RECEIVE_DATA:
 		case RECEIVE_TIMEOUT:
 		case LINE_STATUS:
 		case MODEM_STATUS:
-		;
+			;
 	}
 
-	return 1;
+	return 0;
 }
 
 
@@ -137,22 +122,18 @@ int com1_intr_handler()
  */
 int com2_intr_handler()
 {
-/***************************************/
-	printk("Interrupt IRQ3\n");
-/***************************************/
 	switch(inb(reg[1].iir)&0xf)
 	{
 		case TRANSMIT_EMPTY:
-			wake_intr(&wait_intr_queue[1],TASK_DEVICE_WAIT);
-			break;
+			return wake_intr(&wait_intr_queue[1],TASK_DEVICE_WAIT);
 		case RECEIVE_DATA:
 		case RECEIVE_TIMEOUT:
 		case LINE_STATUS:
 		case MODEM_STATUS:
-		;
+			;
 	}
 
-	return 1;
+	return 0;
 }
 
 
@@ -305,6 +286,69 @@ int write_com(int com,char *buf,size_t count)
 }
 
 
+/************************************************************************************************
+ *
+ * System call interface
+ *
+ ************************************************************************************************/
+
+
+static int open0(dev_t dev, int oflags, int devtype, d_thread_t *proc)
+{
+	return 0;
+}
+
+static int open1(dev_t dev, int oflags, int devtype, d_thread_t *proc)
+{
+	return 0;
+}
+
+static int close0(dev_t dev, int fflag, int devtype, d_thread_t *proc)
+{
+	return 0;
+}
+
+static int close1(dev_t dev, int fflag, int devtype, d_thread_t *proc)
+{
+	return 0;
+}
+
+static int read0(dev_t dev, struct uio *uio, int ioflag)
+{
+	return 0;
+}
+
+static int read1(dev_t dev, struct uio *uio, int ioflag)
+{
+	return 0;
+}
+
+static int write_ttys0(dev_t dev, struct uio *uio, int ioflag)
+{
+	return write_ttys(0, (const char*)uio->uio_iov->iov_base, uio->uio_iov->iov_len);
+}
+
+static int write_ttys1(dev_t dev, struct uio *uio, int ioflag)
+{
+	return write_ttys(1, (const char*)uio->uio_iov->iov_base, uio->uio_iov->iov_len);
+}
+
+static int ioctl_ttys0(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
+{
+	return 0;
+}
+
+static int ioctl_ttys1(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
+{
+	return 0;
+}
+
+static int poll(dev_t dev, int events, d_thread_t *p)
+{
+	return POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM;
+}
+
+
 /************************************************************************************************
  *
  * Init COM
@@ -312,12 +356,43 @@ int write_com(int com,char *buf,size_t count)
  ************************************************************************************************/
 
 
+static struct cdevsw com_info[2]={
+	{open0, 
+	close0, 
+	read0, 
+	write_ttys0, 
+	ioctl_ttys0, 
+	poll, 
+	NULL, 
+	NULL, 
+	"ttys0", 
+	0,
+	NULL, 
+	NULL, 
+	D_TTY,
+},
+	{open1, 
+	close1, 
+	read1, 
+	write_ttys1, 
+	ioctl_ttys1, 
+	poll, 
+	NULL, 
+	NULL, 
+	"ttys1",
+	0,
+	NULL, 
+	NULL, 
+	D_TTY,
+}
+};
+
+
 void init_com()
 {
 	int i;
 	COM_CONF conf;
 
-
 	conf.lcr=0x3;		/* 8bit,1stop bit,no parity(8N1) */
 	conf.boud=B38400;
 	conf.inbuf=NULL;
@@ -328,7 +403,7 @@ void init_com()
 	{
 		if((uart[i]=detect_uart(i))==0)continue;
 		if(conf_com(i,&conf)!=0)continue;	/* Init com */
-		regist_device(&com_info[0]);		/* Regster to dev filesystem */
+		make_dev(&com_info[0], 0, 0, 0, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, "ttys");
 	}
 }
 
@@ -383,36 +458,6 @@ void set_com_intr()
 		if(uart[i]>0)printk("ttys%d : %s\n",i,uart_name[uart[i]]);
 }
 
-
-/************************************************************************************************
- *
- * System call interface
- *
- ************************************************************************************************/
-
-
-/*
- * Write call
- */
-int write_ttys0(void *buf,size_t count,size_t x)
-{
-	return write_ttys(0,(const char*)buf,count);
-}
-
-int write_ttys1(void *buf,size_t count,size_t x)
-{
-	return write_ttys(1,(const char*)buf,count);
-}
-
-int ioctl_ttys0(int prm_num,int cmd,...)
-{
-	return 0;
-}
-
-int ioctl_ttys1(int prm_num,int cmd,...)
-{
-	return 0;
-}
 /***************************************************************************************/
 void test_com()
 {
