@@ -9,9 +9,11 @@
 
 
 #include<types.h>
+#include<sys/param.h>
+#include<sys/module.h>
 #include<lib.h>
 #include<apm.h>
-#include<except.h>
+#include<except_i386.h>
 #include<sp.h>
 #include<mp.h>
 #include<mm.h>
@@ -116,40 +118,15 @@ void start_tasking()
  *
  *************************************************************************************************/
 
-/*
- * 顼
- * parameters : error message
- */
-static void doError(const char *func,int errno)
-{
-	print_error(errno,func);
-	idle();
-}
-
 
 void main()
 {
-	int rest;
-
-
 	/* 㳰ϥɥꤹ롣 */
 	init_except();
 
-	/* Init tty. */
+	/* Init console. */
 	initConsole();
-	init_com();
-	switch(TTY)
-	{
-		case 1:
-			write_tty=writeConsole;
-			break;
-		case 2:
-			write_tty=write_com1;
-			break;
-		case 3:
-			write_tty=write_com2;
-			break;
-	}
+	write_tty=writeConsole;
 
 	/* TSSǥץɤ롣 */
 	set_tss(TSS_DES);
@@ -160,11 +137,17 @@ void main()
 	/* cpuեߥ꡼Υå */
 	cpu_family=(cpuid()>>8)&0xf;
 	if(cpu_family<CPU_FAMILY_PENTIUM)
-		doError("This cpu is not supported!",0);
+	{
+		PRINT_ERR("This cpu is not supported!");
+		idle();
+	}
 
 	/* ɥ쥹饤20ӥåܤͭˤ롣 */
 	if(enable_a20()==-1)
-		doError("A20 can't be enable. Boot stop!",0);
+	{
+		PRINT_ERR("A20 can't be enable. Boot stop!");
+		idle();
+	}
 
 	/* Apmν */
 	InitApm();
@@ -178,12 +161,18 @@ void main()
 	/* cpuåͤ¬ꡣ */
 	printk("cpu0 clock = %d HZ\n",count_cpu_clock());
 
+	// ʥν
+	initSignal();
+
 	/* ɥץ롣 */
-	if((rest=set_idle_proc(0))<0)
-		doError("set_idle_proc",-rest);
+	if (set_idle_proc(0) < 0)
+	{
+		PRINT_ERR("set_idle_proc error!");
+		idle();
+	}
 
 	/* ߤSMP롣 */
-	if(MFPS_addres)
+	if (MFPS_addres)
 	{
 		if(init_mp()==-1)idle();
 	}
@@ -195,45 +184,52 @@ void main()
 
 
 /* ǽΥ桼ץ¹ */
-extern int startExec(int,const char*,void*,void*);
+extern int startExec(const char*,void*,void*);
 
 
 void init()
 {
 	int rest;
 
-
 	/* ߳ϡ */
 	sti();
 
 	/* ǽΥץ process0 롣 */
-    {
-		int error;
-
-		error=sys_fork();
-    	if(error<0)
-    	{
-    		printk("Faile sys_fork!\n");
-    		idle();
-    	}
-    	else if(error>0)
-    		/* ɥץ򥹥꡼פ롣 */
-			start_tasking();
+	rest = sys_fork();
+	if (rest < 0){
+		PRINT_ERR("Process0 fork() error!");
+		idle();
 	}
+	else if (0 < rest){
+		/* ɥץ򥹥꡼פ롣 */
+		start_tasking();
+	}
+
+	/* Init time. */
+	init_time();
 
 	/* Init file system. */
 	init_fs();
 
-	/* Init device system. */
-	if((rest=init_device())<0)
-		doError("init_device",-rest);
+	/* ǥХե륷ƥν */
+	initDeviceFs();
+
+	/* init serial. */
+	init_com();
+
+	/* ֥ååν */
+	initBlockCache();
+
+	/* ǥХե륷ƥΥޥ */
+	if (mountDeviceFs() < 0)
+	{
+		PRINT_ERR("mountDeviceFs() error!");
+		idle();
+	}
 
 	/* Init PCI. */
 	initPci();
 
-	/* Init time. */
-	init_time();
-
 	/* Set COM interrupt. */
 	set_com_intr();
 
@@ -247,57 +243,91 @@ void init()
 
 	/* 󥽡ΥǥХϿ */
 	if((rest=registConsole())<0)
-		doError("registConsole",-rest);
+	{
+		PRINT_ERR("registConsole error!");
+		idle();
+	}
 
 	/* NULLǥХϿ */
 	if((rest=registNullDev())<0)
-		doError("registNullDev",-rest);
+	{
+		PRINT_ERR("registNullDev error!");
+		idle();
+	}
 
 	/* EXT2ե륷ƥν */
 	if((rest=initExt2tFs())<0)
-		doError("initExt2tFs",-rest);
+	{
+		PRINT_ERR("initExt2tFs error!");
+		idle();
+	}
 
 	/* ڡѵΰν */
-	if((rest=initBackupDev(PAGING_BACKUP_DEVICE))<0)
-		doError("initBackupDev",-rest);
+	if ((rest = initBackupDev(PAGING_BACKUP_DEVICE)) < 0)
+	{
+		PRINT_ERR("initBackupDev error!");
+		idle();
+	}
 	printk("Page backup device : %s\n",PAGING_BACKUP_DEVICE);
 
 	/* mount root filesystem. */
-	if((rest=mountRoot(ROOT_DEVICE,"ext2"))<0)
-		doError("mountRoot",-rest);
+	if((rest=mountRoot(ROOT_DEVICE,"ext2")) < 0)
+	{
+		PRINT_ERR("mountRoot error!");
+		idle();
+	}
 	printk("Root device : %s\nRoot file system : ext2\n",ROOT_DEVICE);
 
 	/* Init keyboard driver. */
 	initKeyboard();
 
-/************** ͥåȥطν *******************************************/
-	/* Init network.  */
+	/* ͥåȥν  */
 	initNet();
 
-	/* Init ethernet driver. */
+	/* ͥåȥɥ饤Сν */
 	if((rest=initEther())<0)
-		doError("initEther",-rest);
-/***********************************************************************************/
+	{
+		PRINT_ERR("initEther error!");
+		idle();
+	}
+
+//************************************************************************************
+	// ƥȥɥ饤СȤ߹
+	{
+		extern int echoInitDevice(struct module *, int);
+		rest = echoInitDevice(NULL, MOD_LOAD);
+		if (rest < 0){
+			printk("Failed initialize echo driver!\n");
+		}
+	}
+//************************************************************************************
+	
 	/* Init process0. */
-	if((rest=initProc1())<0)
-		doError("initProc0",-rest);
+	if ((rest = initProc1())<0)
+	{
+		PRINT_ERR("initProc0 error!");
+		idle();
+	}
 
 	/* ǽΥ桼ץư롣 */
-	rest=sys_fork();
-   	if(rest<0)
+	rest = sys_fork();
+   	if (rest < 0)
    	{
-   		doError("sys_fork",-rest);
+   		PRINT_ERR("Fork of first user process error!");
    		idle();
    	}
-   	else if(rest==0)
+   	else if (rest == 0)
    	{
    		/* 桼ץ¹Ԥ롣 */
-		if((rest=startExec(SYS_EXEC,"/shell",NULL,NULL))<0)
-			doError("startExec",-rest);
+// 		rest = startExec("/shell",NULL,NULL);
+   		rest = startExec("/sbin/init",NULL,NULL);
+		if (rest < 0)
+		{
+			PRINT_ERR("startExec error!");
+			idle();
+		}
 	}
-/***********************************/
-idle();
-/***********************************/
+
 	/* ҥץexitԤ */
 	waitProc1();
 }
@@ -320,7 +350,8 @@ static struct SYSUTS{
 
 int sys_uname(struct utsname *buf)
 {
-	if(checkMem(buf)==-1)return -EFAULT;
+	if (checkMem(buf,sizeof(struct utsname)) == -1)
+		return -EFAULT;
 
 	memcpy(buf->sysname,sysuts.sysname,strlen(sysuts.sysname)+1);
 	memcpy(buf->nodename,sysuts.nodename,strlen(sysuts.nodename)+1);
@@ -330,3 +361,9 @@ int sys_uname(struct utsname *buf)
 
 	return 0;
 }
+
+/****************************************************************************************************/
+void test_setup()
+{
+	printk("ok\n");
+}
