@@ -19,33 +19,33 @@
 
 /* ʥ빽¤Ρ */
 typedef struct{
-	SIG_ACTION action[SIG_ACTIONS];	/* sigunal actions. */
-	uint actMask[SIG_ACTIONS];		/* ϥɥ顼¹Υޥ */
-	uint signal;					/* signal bitmap. */
-	uint reserve;					/* reserve signal bitmap. */
-	uint mask;						/* signal mask. */
-	uint beforeEsp;					/* 桼ϥɥܹΥͥesp */
-	uint beforeStackEntry;			/* 桼ϥɥܹΥͥ륹åȥ꡼ */
-	int lock;						/* ʥԤƱå */
+	SIG_ACTION	 action[SIG_ACTIONS];	/* sigunal actions. */
+	sigset_t	 actMask[SIG_ACTIONS];	/* ϥɥ顼¹Υޥ */
+	int			 actFlag[SIG_ACTIONS];	/* ϥɥ顼¹ԥե饰 */
+	uint		 signal;				/* signal bitmap. */
+	uint		 reserve;				/* reserve signal bitmap. */
+	uint		 mask;					/* signal mask. */
+	int			 lock;					/* ʥԤƱå */
 }SIGNAL;
 
 
-extern void doSignal(uint *user_esp);
+extern void doSignal();
+extern void doSignalRestart();
 extern int sendSignal(PROC*,int);
-extern int forceSendSignal(PROC*,int);
+extern int forceSendSignal(PROC*, const int, const int);
 extern void sigWait();
 extern SIGNAL *setSignal(SIGNAL*);
 extern void releaseSignal(SIGNAL*);
 extern void resetSignal(SIGNAL*);
-extern uint getBeforeEsp();
-extern uint getBeforeStackEntry();
 extern int isSigint();
-extern SIGNAL *initSignal();
+extern void setSigAllMask();
+extern SIGNAL *setFirstSignal();
 extern int sys_sigaction();
 extern int sys_sigprocmask();
 extern int sys_kill();
 extern int sys_sigpending();
 extern int sys_sigsuspend();
+extern void initSignal();
 
 
 #endif
