@@ -21,17 +21,18 @@ extern void test_fd();
 extern void test_com();
 extern void test_time(uint);
 extern void test_device();
-extern void test_fs(PROC*);
+extern int test_fs();
 extern void test_orig_fs();
 extern void test_fatFs();
 extern void test_elf(void*);
 extern int test_ext2(int);
-extern int test_mm(uint*);
+extern int test_mm(uint);
 extern void test_proc();
 extern void test_pci();
 extern int test_arp();
 extern void test_tcp();
 extern void test_mp();
+extern void testSignal();
 
 
 #endif
