@@ -12,17 +12,17 @@
 #define time_h
 
 
-#define KERNEL 
-
-
 #include<lib.h>
-#include<sys/select.h>
+#include<sys/time.h>
 
 
 #ifndef ASM_FILE
 
 
 enum{
+	TIMER_MAX = 2,		/* ޡ */
+	ITIMER_MAX = 2,		/* itimer */
+
 	PIT_HZ=1193181,		/* 8254󥿡Х륿ޡΥإĿ */
 
 	/* 8254 interval timer IO register */
@@ -49,10 +49,24 @@ enum{
 typedef struct TIMER{
 	struct TIMER *next;
 	struct TIMER *prev;
-	uint count;
+	uint init;						/* ޡ */
+	uint count;						/* Ĥ */
 	int flag;
+	struct TIMER *link;				/* ³ */
+	void *proc;						/* ץ */
+	void (*handle)(struct TIMER*);	/* ޡϥɥ顼 */
 }TIMER;
 
+typedef struct{
+	int64 init;			/* ޡ͡rdtsc͡ */
+	int64 cur;			/* Ĥ֡rdtsc͡ */
+}ITIMER;
+
+enum{
+	/* itimerʬ */
+	ITIME_SYS = 0,		/* itimer󥷥ƥ */
+	ITIME_PRC = 1,		/* itimer桼 */
+};
 
 extern int clock_1micro;
 extern int clock_1m;
@@ -75,19 +89,23 @@ extern inline void write_cmos(int addr,uchar value)
 }
 
 extern int count_cpu_clock();
-extern int set_timer(uint);
-extern void del_timer(TIMER*);
-extern void setTimerStruct(void*);
-extern void releaseTimer(TIMER*,TIMER*);
+extern int setWake(uint);
+extern void delWakeTimer(TIMER*);
+extern void initTimer(TIMER*, ITIMER*);
+extern void releaseTimer(TIMER*);
 extern void init_time();
 extern void calcUserTime();
 extern void calcSysTime();
+extern void doItimer(ITIMER*,uint64,int);
+extern void gettimeofday(struct timeval*);
 extern int sys_time();
 extern int sys_sleep();
 extern void sys_wake(void*);
 extern int sys_alarm();
 extern int sys_settime(uint);
 extern int sys_gettimeofday();
+extern int sys_getitimer(int which, struct itimerval *value);
+extern int sys_setitimer(int which, const struct itimerval *value, struct itimerval *ovalue);
 
 
 #endif
