@@ -13,19 +13,28 @@
 
 
 #include<sys/types.h>
-#include<share/stdint.h>
 
 
 #ifndef ASM_FILE
 
 
-/* ȼ */
-typedef int				   bool;
-typedef long long		   int64;
-typedef unsigned long long uint64;
+enum{
+	NOERR	= 0,
+	ERR		= -1,
+	FALSE	= 0,
+	TRUE	= 1,
+	YES		= 1,
+	NO		= 0,
+};
 
 
-#define NULL ((void *)0)
+typedef int					loff_t;
+typedef long long			int64;
+typedef unsigned long long	uint64;
+
+// Linux driver
+typedef unsigned char	u8;
+typedef unsigned short	u16;
 
 
 #endif
