@@ -32,105 +32,90 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * 2004: modified by minoru murashma.
  */
 
+#ifndef lint
+/*static char sccsid[] = "@(#)function.c	8.6 (Berkeley) 4/1/94";*/
+#endif /* not lint */
 
+#include <sys/param.h>
+#include <sys/ucred.h>
 #include <sys/stat.h>
-#include <sys/statvfs.h>
 #include <sys/wait.h>
+#include <sys/mount.h>
+
+#include <err.h>
 #include <errno.h>
 #include <fnmatch.h>
+#include <fts.h>
 #include <grp.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "../fts.h"
-#include "../tzfile.h"
-#include "../ucred.h"
-#include "../err.h"
-#include "../mount.h"
+
 #include "find.h"
 
-
-#define	COMPARE(a, b) {		\
-	switch (plan->flags) {	\
-	case F_EQUAL:			\
-		return (a == b);	\
-	case F_LESSTHAN:		\
-		return (a < b);		\
-	case F_GREATER:			\
-		return (a > b);		\
-	default:				\
-		abort();			\
-		return 0;			\
-	}						\
-}
-
-
-static PLAN *palloc(enum ntype t,int (*f)(PLAN *, FTSENT *))
-{
-	PLAN *new;
-
-	
-	if ((new = malloc(sizeof(PLAN))) == NULL)
-		err(1, NULL);
-	new->type = t;
-	new->eval = f;
-	new->flags = 0;
-	new->next = NULL;
-	
-	return (new);
+#define	COMPARE(a, b) {							\
+	switch (plan->flags) {						\
+	case F_EQUAL:							\
+		return (a == b);					\
+	case F_LESSTHAN:						\
+		return (a < b);						\
+	case F_GREATER:							\
+		return (a > b);						\
+	default:							\
+		abort();						\
+		return 0;					\
+	}								\
 }
 
+static PLAN *palloc __P((enum ntype, int (*) __P((PLAN *, FTSENT *))));
 
 /*
  * find_parsenum --
  *	Parse a string of the form [+-]# and return the value.
  */
-static long find_parsenum(PLAN *plan,char *option,char *vp,char *endch)
+static long long
+find_parsenum(plan, option, vp, endch)
+	PLAN *plan;
+	char *option, *vp, *endch;
 {
-	long value;
+	long long value;
 	char *endchar, *str;	/* Pointer to character ending conversion. */
-    
-	
+
 	/* Determine comparison from leading + or -. */
 	str = vp;
-	switch (*str) 
-	{
-		case '+':
-			++str;
-			plan->flags = F_GREATER;
-			break;
-		case '-':
-			++str;
-			plan->flags = F_LESSTHAN;
-			break;
-		default:
-			plan->flags = F_EQUAL;
-			break;
+	switch (*str) {
+	case '+':
+		++str;
+		plan->flags = F_GREATER;
+		break;
+	case '-':
+		++str;
+		plan->flags = F_LESSTHAN;
+		break;
+	default:
+		plan->flags = F_EQUAL;
+		break;
 	}
-    
+
 	/*
-	 * Convert the string with strtol().  Note, if strtol() returns zero
+	 * Convert the string with strtoq().  Note, if strtoq() returns zero
 	 * and endchar points to the beginning of the string we know we have
 	 * a syntax error.
 	 */
-	value = strtol(str, &endchar, 10);
+	value = strtoq(str, &endchar, 10);
 	if (value == 0 && endchar == str)
 		errx(1, "%s: %s: illegal numeric value", option, vp);
 	if (endchar[0] && (endch == NULL || endchar[0] != *endch))
 		errx(1, "%s: %s: illegal trailing character", option, vp);
 	if (endch)
 		*endch = endchar[0];
-	
 	return (value);
 }
 
-
 /*
  * The value of n for the inode times (atime, ctime, and mtime) is a range,
  * i.e. n matches from (n - 1) to n 24 hour periods.  This interacts with
@@ -141,66 +126,67 @@ static long find_parsenum(PLAN *plan,char *option,char *vp,char *endch)
 	if ((p)->type == ttype && (p)->flags == F_LESSTHAN)		\
 		++((p)->t_data);
 
-
 /*
  * -atime n functions --
  *
  *	True if the difference between the file access time and the
  *	current time is n 24 hour periods.
  */
-int f_atime(PLAN *plan,FTSENT *entry)
+int
+f_atime(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	extern time_t now;
 
-	
-	COMPARE((now - entry->fts_statp->st_atime + SECSPERDAY - 1) / SECSPERDAY, plan->t_data);
+	COMPARE((now - entry->fts_statp->st_atime +
+	    86400 - 1) / 86400, plan->t_data);
 }
 
-PLAN * c_atime(char *arg)
+PLAN *
+c_atime(arg)
+	char *arg;
 {
 	PLAN *new;
 
-	
 	ftsoptions &= ~FTS_NOSTAT;
 
 	new = palloc(N_ATIME, f_atime);
 	new->t_data = find_parsenum(new, "-atime", arg, NULL);
 	TIME_CORRECT(new, N_ATIME);
-	
 	return (new);
 }
-
-
 /*
  * -ctime n functions --
  *
- * True if the difference between the last change of file
- * status information and the current time is n 24 hour periods.
+ *	True if the difference between the last change of file
+ *	status information and the current time is n 24 hour periods.
  */
-int f_ctime(PLAN *plan,FTSENT *entry)
+int
+f_ctime(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	extern time_t now;
 
-	
-	COMPARE((now - entry->fts_statp->st_ctime + SECSPERDAY - 1) / SECSPERDAY, plan->t_data);
+	COMPARE((now - entry->fts_statp->st_ctime +
+	    86400 - 1) / 86400, plan->t_data);
 }
 
-
-PLAN * c_ctime(char *arg)
+PLAN *
+c_ctime(arg)
+	char *arg;
 {
 	PLAN *new;
 
-	
 	ftsoptions &= ~FTS_NOSTAT;
 
 	new = palloc(N_CTIME, f_ctime);
 	new->t_data = find_parsenum(new, "-ctime", arg, NULL);
 	TIME_CORRECT(new, N_CTIME);
-	
 	return (new);
 }
 
-
 /*
  * -depth functions --
  *
@@ -208,21 +194,22 @@ PLAN * c_ctime(char *arg)
  *	so that all entries in a directory are acted on before the directory
  *	itself.
  */
-int f_always_true(PLAN *plan,FTSENT *entry)
+int
+f_always_true(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return (1);
 }
 
-
-PLAN *c_depth()
+PLAN *
+c_depth()
 {
 	isdepth = 1;
 
-
 	return (palloc(N_DEPTH, f_always_true));
 }
 
-
 /*
  * [-exec | -ok] utility [arg ... ] ; functions --
  *
@@ -235,41 +222,43 @@ PLAN *c_depth()
  *	The primary -ok is different in that it requests affirmation of the
  *	user before executing the utility.
  */
-int f_exec(PLAN *plan,FTSENT *entry)
+int
+f_exec(plan, entry)
+	register PLAN *plan;
+	FTSENT *entry;
 {
 	extern int dotfd;
-	int cnt;
+	register int cnt;
 	pid_t pid;
 	int status;
 
-	
 	for (cnt = 0; plan->e_argv[cnt]; ++cnt)
 		if (plan->e_len[cnt])
-			brace_subst(plan->e_orig[cnt], &plan->e_argv[cnt],entry->fts_path, plan->e_len[cnt]);
+			brace_subst(plan->e_orig[cnt], &plan->e_argv[cnt],
+			    entry->fts_path, plan->e_len[cnt]);
 
 	if (plan->flags == F_NEEDOK && !queryuser(plan->e_argv))
 		return (0);
 
-	switch (pid = fork()) 
-	{
-		case -1:
-			err(1, "fork");
-			/* NOTREACHED */
-		case 0:
-			if (fchdir(dotfd)) 
-			{
-				warn("chdir");
-				_exit(1);
-			}
-			execvp(plan->e_argv[0], plan->e_argv);
-			warn("%s", plan->e_argv[0]);
+	/* make sure find output is interspersed correctly with subprocesses */
+	fflush(stdout);
+
+	switch (pid = vfork()) {
+	case -1:
+		err(1, "fork");
+		/* NOTREACHED */
+	case 0:
+		if (fchdir(dotfd)) {
+			warn("chdir");
 			_exit(1);
+		}
+		execvp(plan->e_argv[0], plan->e_argv);
+		warn("%s", plan->e_argv[0]);
+		_exit(1);
 	}
 	pid = waitpid(pid, &status, 0);
-	
 	return (pid != -1 && WIFEXITED(status) && !WEXITSTATUS(status));
 }
- 
 
 /*
  * c_exec --
@@ -278,23 +267,25 @@ int f_exec(PLAN *plan,FTSENT *entry)
  *	argv array, and one with integer values that are lengths of the
  *	strings, but also flags meaning that the string has to be massaged.
  */
-PLAN *c_exec(char ***argvp,int isok)
+PLAN *
+c_exec(argvp, isok)
+	char ***argvp;
+	int isok;
 {
 	PLAN *new;			/* node returned */
-	int cnt;
-	char **argv, **ap, *p;
+	register int cnt;
+	register char **argv, **ap, *p;
 
-	
 	isoutput = 1;
-    
+
 	new = palloc(N_EXEC, f_exec);
 	if (isok)
 		new->flags = F_NEEDOK;
 
-	for (ap = argv = *argvp;; ++ap) 
-	{
+	for (ap = argv = *argvp;; ++ap) {
 		if (!*ap)
-			errx(1,"%s: no terminating \";\"", isok ? "-ok" : "-exec");
+			errx(1,
+			    "%s: no terminating \";\"", isok ? "-ok" : "-exec");
 		if (**ap == ';')
 			break;
 	}
@@ -304,18 +295,15 @@ PLAN *c_exec(char ***argvp,int isok)
 	new->e_orig = (char **)emalloc((u_int)cnt * sizeof(char *));
 	new->e_len = (int *)emalloc((u_int)cnt * sizeof(int));
 
-	for (argv = *argvp, cnt = 0; argv < ap; ++argv, ++cnt) 
-	{
+	for (argv = *argvp, cnt = 0; argv < ap; ++argv, ++cnt) {
 		new->e_orig[cnt] = *argv;
 		for (p = *argv; *p; ++p)
-			if (p[0] == '{' && p[1] == '}') 
-			{
-				new->e_argv[cnt] = emalloc((u_int)PATH_MAX);
-				new->e_len[cnt] = PATH_MAX;
+			if (p[0] == '{' && p[1] == '}') {
+				new->e_argv[cnt] = emalloc((u_int)MAXPATHLEN);
+				new->e_len[cnt] = MAXPATHLEN;
 				break;
 			}
-		if (!*p) 
-		{
+		if (!*p) {
 			new->e_argv[cnt] = *argv;
 			new->e_len[cnt] = 0;
 		}
@@ -323,33 +311,33 @@ PLAN *c_exec(char ***argvp,int isok)
 	new->e_argv[cnt] = new->e_orig[cnt] = NULL;
 
 	*argvp = argv + 1;
-	
 	return (new);
 }
- 
- 
+
 /*
  * -follow functions --
  *
  *	Always true, causes symbolic links to be followed on a global
  *	basis.
  */
-PLAN *c_follow()
+PLAN *
+c_follow()
 {
 	ftsoptions &= ~FTS_PHYSICAL;
 	ftsoptions |= FTS_LOGICAL;
 
-	
 	return (palloc(N_FOLLOW, f_always_true));
 }
 
-
 /*
  * -fstype functions --
  *
  *	True if the file is of a certain type.
  */
-int f_fstype(PLAN *plan,FTSENT *entry)
+int
+f_fstype(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	static dev_t curdev;	/* need a guaranteed illegal dev value */
 	static int first = 1;
@@ -357,18 +345,16 @@ int f_fstype(PLAN *plan,FTSENT *entry)
 	static short val;
 	char *p, save[2];
 
-	
 	/* Only check when we cross mount point. */
-	if (first || curdev != entry->fts_statp->st_dev) 
-	{
+	if (first || curdev != entry->fts_statp->st_dev) {
 		curdev = entry->fts_statp->st_dev;
 
 		/*
 		 * Statfs follows symlinks; find wants the link's file system,
 		 * not where it points.
 		 */
-		if (entry->fts_info == FTS_SL || entry->fts_info == FTS_SLNONE) 
-		{
+		if (entry->fts_info == FTS_SL ||
+		    entry->fts_info == FTS_SLNONE) {
 			if ((p = strrchr(entry->fts_accpath, '/')) != NULL)
 				++p;
 			else
@@ -377,86 +363,95 @@ int f_fstype(PLAN *plan,FTSENT *entry)
 			p[0] = '.';
 			save[1] = p[1];
 			p[1] = '\0';
-			
-		}
-		else p = NULL;
+
+		} else
+			p = NULL;
 
 		if (statfs(entry->fts_accpath, &sb))
 			err(1, "%s", entry->fts_accpath);
 
-		if (p) 
-		{
+		if (p) {
 			p[0] = save[0];
 			p[1] = save[1];
 		}
 
 		first = 0;
-
-		/*
-		 * Further tests may need both of these values, so
-		 * always copy both of them.
-		 */
-		val = sb.f_flags;
-		val = sb.f_type;
-	}
-	switch (plan->flags) 
-	{
+		switch (plan->flags) {
 		case F_MTFLAG:
-			return (val & plan->mt_data);	
+			val = sb.f_flags;
+			break;
 		case F_MTTYPE:
-			return (val == plan->mt_data);
+			val = sb.f_type;
+			break;
 		default:
 			abort();
-			return 0;
+		}
+	}
+	switch(plan->flags) {
+	case F_MTFLAG:
+		return (val & plan->mt_data);
+	case F_MTTYPE:
+		return (val == plan->mt_data);
+	default:
+		abort();
+		return 0;
 	}
 }
 
-
-PLAN *c_fstype(char *arg)
+PLAN *
+c_fstype(arg)
+	char *arg;
 {
-	PLAN *new;
-   struct statvfs svfs; 
-	
+	register PLAN *new;
+
 	ftsoptions &= ~FTS_NOSTAT;
-    
+
 	new = palloc(N_FSTYPE, f_fstype);
-
-	/*
-	 * Check first for a filesystem name.
-	 */
-	if (statvfs(arg, &svfs) == 0) 
-	{
-		new->flags = F_MTTYPE;
-		new->mt_data = svfs.f_type;
-		return (new);
-	}
-
-	switch (*arg) 
-	{
-		case 'l':
-			if (!strcmp(arg, "local")) 
-			{
-				new->flags = F_MTFLAG;
-				new->mt_data = MNT_LOCAL;
-				return (new);
-			}
-			break;
-		case 'r':
-			if (!strcmp(arg, "rdonly")) 
-			{
-				new->flags = F_MTFLAG;
-				new->mt_data = MNT_RDONLY;
-				return (new);
-			}
-			break;
+	switch (*arg) {
+	case 'l':
+		if (!strcmp(arg, "local")) {
+			new->flags = F_MTFLAG;
+			new->mt_data = MNT_LOCAL;
+			return (new);
+		}
+		break;
+	case 'm':
+		if (!strcmp(arg, "mfs")) {
+			new->flags = F_MTTYPE;
+			new->mt_data = MOUNT_MFS;
+			return (new);
+		} else if (!strcmp(arg, "msdos")) {
+			new->flags = F_MTTYPE;
+			new->mt_data = MOUNT_MSDOS;
+			return (new);
+		}
+		break;
+	case 'n':
+		if (!strcmp(arg, "nfs")) {
+			new->flags = F_MTTYPE;
+			new->mt_data = MOUNT_NFS;
+			return (new);
+		}
+		break;
+	case 'r':
+		if (!strcmp(arg, "rdonly")) {
+			new->flags = F_MTFLAG;
+			new->mt_data = MNT_RDONLY;
+			return (new);
+		}
+		break;
+	case 'u':
+		if (!strcmp(arg, "ufs")) {
+			new->flags = F_MTTYPE;
+			new->mt_data = MOUNT_UFS;
+			return (new);
+		}
+		break;
 	}
 	errx(1, "%s: unknown file type", arg);
 	/* NOTREACHED */
-
-	return NULL;
 }
 
-
 /*
  * -group gname functions --
  *
@@ -464,158 +459,168 @@ PLAN *c_fstype(char *arg)
  *	an equivalent of the getgrnam() function does not return a valid group
  *	name, gname is taken as a group ID.
  */
-int f_group(PLAN *plan,FTSENT *entry)
+int
+f_group(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return (entry->fts_statp->st_gid == plan->g_data);
 }
- 
-PLAN *c_group(char *gname)
+
+PLAN *
+c_group(gname)
+	char *gname;
 {
 	PLAN *new;
 	struct group *g;
 	gid_t gid;
-    
-	
+
 	ftsoptions &= ~FTS_NOSTAT;
 
 	g = getgrnam(gname);
-	if (g == NULL) 
-	{
+	if (g == NULL) {
 		gid = atoi(gname);
 		if (gid == 0 && gname[0] != '0')
 			errx(1, "-group: %s: no such group", gname);
-	} 
-	else gid = g->gr_gid;
-    
+	} else
+		gid = g->gr_gid;
+
 	new = palloc(N_GROUP, f_group);
 	new->g_data = gid;
-	
 	return (new);
 }
 
-
 /*
  * -inum n functions --
  *
  *	True if the file has inode # n.
  */
-int f_inum(PLAN *plan,FTSENT *entry)
+int
+f_inum(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	COMPARE(entry->fts_statp->st_ino, plan->i_data);
 }
 
-PLAN *c_inum(char *arg)
+PLAN *
+c_inum(arg)
+	char *arg;
 {
 	PLAN *new;
-    
-	
+
 	ftsoptions &= ~FTS_NOSTAT;
-    
+
 	new = palloc(N_INUM, f_inum);
 	new->i_data = find_parsenum(new, "-inum", arg, NULL);
-	
 	return (new);
 }
 
-
 /*
  * -links n functions --
  *
  *	True if the file has n links.
  */
-int f_links(PLAN *plan,FTSENT *entry)
+int
+f_links(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	COMPARE(entry->fts_statp->st_nlink, plan->l_data);
 }
- 
-PLAN *c_links(char *arg)
+
+PLAN *
+c_links(arg)
+	char *arg;
 {
 	PLAN *new;
-    
-	
+
 	ftsoptions &= ~FTS_NOSTAT;
-    
+
 	new = palloc(N_LINKS, f_links);
 	new->l_data = (nlink_t)find_parsenum(new, "-links", arg, NULL);
-	
 	return (new);
 }
 
-
 /*
  * -ls functions --
  *
  *	Always true - prints the current entry to stdout in "ls" format.
  */
-int f_ls(PLAN *plan,FTSENT *entry)
+int
+f_ls(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	printlong(entry->fts_path, entry->fts_accpath, entry->fts_statp);
 	return (1);
 }
- 
-PLAN *c_ls()
+
+PLAN *
+c_ls()
 {
 	ftsoptions &= ~FTS_NOSTAT;
 	isoutput = 1;
-    
-	
+
 	return (palloc(N_LS, f_ls));
 }
 
-
 /*
  * -mtime n functions --
  *
  *	True if the difference between the file modification time and the
  *	current time is n 24 hour periods.
  */
-int f_mtime(PLAN *plan,FTSENT *entry)
+int
+f_mtime(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	extern time_t now;
 
-	
-	COMPARE((now - entry->fts_statp->st_mtime + SECSPERDAY - 1) / SECSPERDAY, plan->t_data);
+	COMPARE((now - entry->fts_statp->st_mtime + 86400 - 1) /
+	    86400, plan->t_data);
 }
 
-
-PLAN *c_mtime(char *arg)
+PLAN *
+c_mtime(arg)
+	char *arg;
 {
 	PLAN *new;
 
-	
 	ftsoptions &= ~FTS_NOSTAT;
 
 	new = palloc(N_MTIME, f_mtime);
 	new->t_data = find_parsenum(new, "-mtime", arg, NULL);
 	TIME_CORRECT(new, N_MTIME);
-	
 	return (new);
 }
 
-
 /*
  * -name functions --
  *
  *	True if the basename of the filename being examined
  *	matches pattern using Pattern Matching Notation S3.14
  */
-int f_name(PLAN *plan,FTSENT *entry)
+int
+f_name(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return (!fnmatch(plan->c_data, entry->fts_name, 0));
 }
- 
-PLAN *c_name(char *pattern)
+
+PLAN *
+c_name(pattern)
+	char *pattern;
 {
 	PLAN *new;
 
-	
 	new = palloc(N_NAME, f_name);
 	new->c_data = pattern;
-	
 	return (new);
 }
 
-
 /*
  * -newer file functions --
  *
@@ -623,96 +628,103 @@ PLAN *c_name(char *pattern)
  *	then the modification time of the file named by the pathname
  *	file.
  */
-int f_newer(PLAN *plan,FTSENT *entry)
+int
+f_newer(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return (entry->fts_statp->st_mtime > plan->t_data);
 }
- 
-PLAN *c_newer(char *filename)
+
+PLAN *
+c_newer(filename)
+	char *filename;
 {
 	PLAN *new;
 	struct stat sb;
-    
-	
+
 	ftsoptions &= ~FTS_NOSTAT;
 
 	if (stat(filename, &sb))
 		err(1, "%s", filename);
 	new = palloc(N_NEWER, f_newer);
 	new->t_data = sb.st_mtime;
-	
 	return (new);
 }
 
-
 /*
  * -nogroup functions --
  *
  *	True if file belongs to a user ID for which the equivalent
  *	of the getgrnam() 9.2.1 [POSIX.1] function returns NULL.
  */
-int f_nogroup(PLAN *plan,FTSENT *entry)
+int
+f_nogroup(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	char *group_from_gid();
 
-	
 	return (group_from_gid(entry->fts_statp->st_gid, 1) ? 0 : 1);
 }
- 
-PLAN *c_nogroup()
+
+PLAN *
+c_nogroup()
 {
 	ftsoptions &= ~FTS_NOSTAT;
 
-	
 	return (palloc(N_NOGROUP, f_nogroup));
 }
 
- 
 /*
  * -nouser functions --
  *
  *	True if file belongs to a user ID for which the equivalent
  *	of the getpwuid() 9.2.2 [POSIX.1] function returns NULL.
  */
-int f_nouser(PLAN *plan,FTSENT *entry)
+int
+f_nouser(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	char *user_from_uid();
 
-	
 	return (user_from_uid(entry->fts_statp->st_uid, 1) ? 0 : 1);
 }
- 
-PLAN * c_nouser()
+
+PLAN *
+c_nouser()
 {
 	ftsoptions &= ~FTS_NOSTAT;
 
 	return (palloc(N_NOUSER, f_nouser));
 }
 
-
 /*
  * -path functions --
  *
  *	True if the path of the filename being examined
  *	matches pattern using Pattern Matching Notation S3.14
  */
-int f_path(PLAN *plan,FTSENT *entry)
+int
+f_path(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return (!fnmatch(plan->c_data, entry->fts_path, 0));
 }
- 
-PLAN * c_path(char *pattern)
+
+PLAN *
+c_path(pattern)
+	char *pattern;
 {
 	PLAN *new;
 
-
 	new = palloc(N_NAME, f_path);
 	new->c_data = pattern;
-
 	return (new);
 }
 
-
 /*
  * -perm functions --
  *
@@ -720,31 +732,34 @@ PLAN * c_path(char *pattern)
  *	with a leading digit, it's treated as an octal mode, otherwise as a
  *	symbolic mode.
  */
-int f_perm(PLAN *plan,FTSENT *entry)
+int
+f_perm(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	mode_t mode;
 
-
-	mode = entry->fts_statp->st_mode & (S_ISUID|S_ISGID|S_ISTXT|S_IRWXU|S_IRWXG|S_IRWXO);
+	mode = entry->fts_statp->st_mode &
+	    (S_ISUID|S_ISGID|S_ISTXT|S_IRWXU|S_IRWXG|S_IRWXO);
 	if (plan->flags == F_ATLEAST)
 		return ((plan->m_data | mode) == mode);
 	else
 		return (mode == plan->m_data);
 	/* NOTREACHED */
 }
- 
-PLAN * c_perm(char *perm)
+
+PLAN *
+c_perm(perm)
+	char *perm;
 {
 	PLAN *new;
 	mode_t *set;
 
-
 	ftsoptions &= ~FTS_NOSTAT;
 
 	new = palloc(N_PERM, f_perm);
 
-	if (*perm == '-')
-	{
+	if (*perm == '-') {
 		new->flags = F_ATLEAST;
 		++perm;
 	}
@@ -753,53 +768,79 @@ PLAN * c_perm(char *perm)
 		err(1, "-perm: %s: illegal mode string", perm);
 
 	new->m_data = getmode(set, 0);
-
 	return (new);
 }
 
-
 /*
  * -print functions --
  *
  *	Always true, causes the current pathame to be written to
  *	standard output.
  */
-int f_print(PLAN *plan,FTSENT *entry)
+int
+f_print(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
-	printf("%s\n", entry->fts_path);
+	(void)puts(entry->fts_path);
 	return (1);
 }
- 
-PLAN * c_print()
+
+PLAN *
+c_print()
 {
 	isoutput = 1;
 
 	return (palloc(N_PRINT, f_print));
 }
 
+/*
+ * -print0 functions --
+ *
+ *	Always true, causes the current pathame to be written to
+ *	standard output followed by a NUL character
+ */
+int
+f_print0(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
+{
+	fputs(entry->fts_path, stdout);
+	fputc('\0', stdout);
+	return (1);
+}
+
+PLAN *
+c_print0()
+{
+	isoutput = 1;
+
+	return (palloc(N_PRINT0, f_print0));
+}
 
 /*
  * -prune functions --
  *
  *	Prune a portion of the hierarchy.
  */
-int f_prune(PLAN *plan,FTSENT *entry)
+int
+f_prune(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	extern FTS *tree;
 
-
 	if (fts_set(tree, entry, FTS_SKIP))
 		err(1, "%s", entry->fts_path);
-
 	return (1);
 }
- 
-PLAN * c_prune()
+
+PLAN *
+c_prune()
 {
 	return (palloc(N_PRUNE, f_prune));
 }
 
-
 /*
  * -size n[c] functions --
  *
@@ -810,20 +851,24 @@ PLAN * c_prune()
 #define	FIND_SIZE	512
 static int divsize = 1;
 
-int f_size(PLAN *plan,FTSENT *entry)
+int
+f_size(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	off_t size;
 
-
-	size = divsize ? (entry->fts_statp->st_size + FIND_SIZE - 1) / FIND_SIZE : entry->fts_statp->st_size;
+	size = divsize ? (entry->fts_statp->st_size + FIND_SIZE - 1) /
+	    FIND_SIZE : entry->fts_statp->st_size;
 	COMPARE(size, plan->o_data);
 }
- 
-PLAN * c_size(char *arg)
+
+PLAN *
+c_size(arg)
+	char *arg;
 {
 	PLAN *new;
 	char endch;
-   
 
 	ftsoptions &= ~FTS_NOSTAT;
 
@@ -832,70 +877,123 @@ PLAN * c_size(char *arg)
 	new->o_data = find_parsenum(new, "-size", arg, &endch);
 	if (endch == 'c')
 		divsize = 0;
-
 	return (new);
 }
 
-
 /*
  * -type c functions --
  *
- *	True if the type of the file is c, where c is b, c, d, p, f or w
- *	for block special file, character special file, directory, FIFO,
- *	regular file or whiteout respectively.
+ *	True if the type of the file is c, where c is b, c, d, p, or f for
+ *	block special file, character special file, directory, FIFO, or
+ *	regular file, respectively.
  */
-int f_type(PLAN *plan,FTSENT *entry)
+int
+f_type(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return ((entry->fts_statp->st_mode & S_IFMT) == plan->m_data);
 }
- 
-PLAN * c_type(char *typestring)
+
+PLAN *
+c_type(typestring)
+	char *typestring;
 {
 	PLAN *new;
 	mode_t  mask;
-   
 
 	ftsoptions &= ~FTS_NOSTAT;
 
-	switch (typestring[0])
-	{
-		case 'b':
-			mask = S_IFBLK;
-			break;
-		case 'c':
-			mask = S_IFCHR;
-			break;
-		case 'd':
-			mask = S_IFDIR;
-			break;
-		case 'f':
-			mask = S_IFREG;
-			break;
-		case 'l':
-			mask = S_IFLNK;
-			break;
-		case 'p':
-			mask = S_IFIFO;
-			break;
-		case 's':
-			mask = S_IFSOCK;
-			break;
-#ifdef FTS_WHITEOUT
-		case 'w':
-			mask = S_IFWHT;
-			ftsoptions |= FTS_WHITEOUT;
-			break;
-#endif /* FTS_WHITEOUT */
-		default:
-			errx(1, "-type: %s: unknown type", typestring);
+	switch (typestring[0]) {
+	case 'b':
+		mask = S_IFBLK;
+		break;
+	case 'c':
+		mask = S_IFCHR;
+		break;
+	case 'd':
+		mask = S_IFDIR;
+		break;
+	case 'f':
+		mask = S_IFREG;
+		break;
+	case 'l':
+		mask = S_IFLNK;
+		break;
+	case 'p':
+		mask = S_IFIFO;
+		break;
+	case 's':
+		mask = S_IFSOCK;
+		break;
+	default:
+		errx(1, "-type: %s: unknown type", typestring);
 	}
-    
+
 	new = palloc(N_TYPE, f_type);
 	new->m_data = mask;
-
 	return (new);
 }
 
+/*
+ * -delete functions --
+ *
+ *	True always.  Makes it's best shot and continues on regardless.
+ */
+int
+f_delete(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
+{
+	/* ignore these from fts */
+	if (strcmp(entry->fts_accpath, ".") == 0 ||
+	    strcmp(entry->fts_accpath, "..") == 0)
+		return (1);
+
+	/* sanity check */
+	if (isdepth == 0 ||			/* depth off */
+	    (ftsoptions & FTS_NOSTAT) ||	/* not stat()ing */
+	    !(ftsoptions & FTS_PHYSICAL) ||	/* physical off */
+	    (ftsoptions & FTS_LOGICAL))		/* or finally, logical on */
+		errx(1, "-delete: insecure options got turned on");
+
+	/* Potentially unsafe - do not accept relative paths whatsoever */
+	if (strchr(entry->fts_accpath, '/') != NULL)
+		errx(1, "-delete: %s: relative path potentially not safe",
+			entry->fts_accpath);
+
+	/* Turn off user immutable bits if running as root */
+	if ((entry->fts_statp->st_flags & (UF_APPEND|UF_IMMUTABLE)) &&
+	    !(entry->fts_statp->st_flags & (SF_APPEND|SF_IMMUTABLE)) &&
+	    geteuid() == 0)
+		chflags(entry->fts_accpath,
+		       entry->fts_statp->st_flags &= ~(UF_APPEND|UF_IMMUTABLE));
+
+	/* rmdir directories, unlink everything else */
+	if (S_ISDIR(entry->fts_statp->st_mode)) {
+		if (rmdir(entry->fts_accpath) < 0 && errno != ENOTEMPTY)
+			warn("-delete: rmdir(%s)", entry->fts_path);
+	} else {
+		if (unlink(entry->fts_accpath) < 0)
+			warn("-delete: unlink(%s)", entry->fts_path);
+	}
+
+	/* "succeed" */
+	return (1);
+}
+
+PLAN *
+c_delete()
+{
+
+	ftsoptions &= ~FTS_NOSTAT;	/* no optimise */
+	ftsoptions |= FTS_PHYSICAL;	/* disable -follow */
+	ftsoptions &= ~FTS_LOGICAL;	/* disable -follow */
+	isoutput = 1;			/* possible output */
+	isdepth = 1;			/* -depth implied */
+
+	return (palloc(N_DELETE, f_delete));
+}
 
 /*
  * -user uname functions --
@@ -904,127 +1002,153 @@ PLAN * c_type(char *typestring)
  *	an equivalent of the getpwnam() S9.2.2 [POSIX.1] function does not
  *	return a valid user name, uname is taken as a user ID.
  */
-int f_user(PLAN *plan,FTSENT *entry)
+int
+f_user(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
 	return (entry->fts_statp->st_uid == plan->u_data);
 }
- 
-PLAN *c_user(char *username)
+
+PLAN *
+c_user(username)
+	char *username;
 {
 	PLAN *new;
 	struct passwd *p;
 	uid_t uid;
-   
 
 	ftsoptions &= ~FTS_NOSTAT;
 
 	p = getpwnam(username);
-	if (p == NULL)
-	{
+	if (p == NULL) {
 		uid = atoi(username);
 		if (uid == 0 && username[0] != '0')
 			errx(1, "-user: %s: no such user", username);
-	}
-	else uid = p->pw_uid;
+	} else
+		uid = p->pw_uid;
 
 	new = palloc(N_USER, f_user);
 	new->u_data = uid;
-
 	return (new);
 }
 
-
 /*
  * -xdev functions --
  *
  *	Always true, causes find not to decend past directories that have a
  *	different device ID (st_dev, see stat() S5.6.2 [POSIX.1])
  */
-PLAN *c_xdev()
+PLAN *
+c_xdev()
 {
 	ftsoptions |= FTS_XDEV;
 
 	return (palloc(N_XDEV, f_always_true));
 }
 
-
 /*
  * ( expression ) functions --
  *
  *	True if expression is true.
  */
-int f_expr(PLAN *plan,FTSENT *entry)
+int
+f_expr(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
-	PLAN *p;
-	int state;
-
-
-	for (p = plan->p_data[0]; p && (state = (p->eval)(p, entry)); p = p->next);
+	register PLAN *p;
+	register int state = 0;
 
+	for (p = plan->p_data[0];
+	    p && (state = (p->eval)(p, entry)); p = p->next);
 	return (state);
 }
 
-
 /*
  * N_OPENPAREN and N_CLOSEPAREN nodes are temporary place markers.  They are
  * eliminated during phase 2 of find_formplan() --- the '(' node is converted
  * to a N_EXPR node containing the expression and the ')' node is discarded.
  */
-PLAN * c_openparen()
+PLAN *
+c_openparen()
 {
 	return (palloc(N_OPENPAREN, (int (*)())-1));
 }
- 
-PLAN * c_closeparen()
+
+PLAN *
+c_closeparen()
 {
 	return (palloc(N_CLOSEPAREN, (int (*)())-1));
 }
 
-
 /*
  * ! expression functions --
  *
  *	Negation of a primary; the unary NOT operator.
  */
-int f_not(PLAN *plan,FTSENT *entry)
+int
+f_not(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
-	PLAN *p;
-	int state;
-
-
-	for (p = plan->p_data[0]; p && (state = (p->eval)(p, entry)); p = p->next);
+	register PLAN *p;
+	register int state = 0;
 
+	for (p = plan->p_data[0];
+	    p && (state = (p->eval)(p, entry)); p = p->next);
 	return (!state);
 }
- 
-PLAN * c_not()
+
+PLAN *
+c_not()
 {
 	return (palloc(N_NOT, f_not));
 }
 
-
 /*
  * expression -o expression functions --
  *
  *	Alternation of primaries; the OR operator.  The second expression is
  * not evaluated if the first expression is true.
  */
-int f_or(PLAN *plan,FTSENT *entry)
+int
+f_or(plan, entry)
+	PLAN *plan;
+	FTSENT *entry;
 {
-	PLAN *p;
-	int state;
+	register PLAN *p;
+	register int state = 0;
 
+	for (p = plan->p_data[0];
+	    p && (state = (p->eval)(p, entry)); p = p->next);
 
-	for (p = plan->p_data[0]; p && (state = (p->eval)(p, entry)); p = p->next);
-
-	if (state) return (1);
-
-	for (p = plan->p_data[1]; p && (state = (p->eval)(p, entry)); p = p->next);
+	if (state)
+		return (1);
 
+	for (p = plan->p_data[1];
+	    p && (state = (p->eval)(p, entry)); p = p->next);
 	return (state);
 }
 
-PLAN * c_or()
+PLAN *
+c_or()
 {
 	return (palloc(N_OR, f_or));
 }
+
+static PLAN *
+palloc(t, f)
+	enum ntype t;
+	int (*f) __P((PLAN *, FTSENT *));
+{
+	PLAN *new;
+
+	if ((new = malloc(sizeof(PLAN))) == NULL)
+		err(1, NULL);
+	new->type = t;
+	new->eval = f;
+	new->flags = 0;
+	new->next = NULL;
+	return (new);
+}
