@@ -89,7 +89,7 @@ struct addrinfo{
 	int               ai_family;	/* Address family of socket. */
 	int               ai_socktype;	/* Socket type. */
 	int               ai_protocol;	/* Protocol of socket. */
-	socklen_t         ai_addrlen;	/* Length of socket address. */
+	int         ai_addrlen;	/* Length of socket address. */
 	struct sockaddr  *ai_addr;		/* Socket address of socket. */
 	char             *ai_canonname;	/* Canonical name of service location. */
 	struct addrinfo  *ai_next;		/* Pointer to next in list. */
@@ -108,7 +108,7 @@ const char      *gai_strerror(int);
 int              getaddrinfo(const char *, const char *,const struct addrinfo *,struct addrinfo **);
 struct hostent	*gethostbyaddr(const char *, int, int);
 struct hostent  *gethostent(void);
-int              getnameinfo(const struct sockaddr *, socklen_t,char *, socklen_t, char *,socklen_t, unsigned);
+int              getnameinfo(const struct sockaddr *, int,char *, int, char *,int, unsigned);
 struct netent   *getnetbyaddr(uint32_t, int);
 struct netent   *getnetbyname(const char *);
 struct netent   *getnetent(void);
@@ -160,5 +160,7 @@ void	_setnetdnsent (int);
 void	_endnetdnsent (void);
 
 void herror();
+const char	*hstrerror(int);
+
 
 #endif
