@@ -10,7 +10,7 @@
 #define share_segment_h
 
 
-#include<share/config.h>
+#include<sys/config.h>
 
 
 #define	KERNEL_CODE_DES 0x8					/* ͥ륳ɥǥץ͡ */
