@@ -7,7 +7,7 @@
 #define stdint_h
 
 
-#include<share/stdint.h>
+#include<sys/stdint.h>
 
 
 #endif
