@@ -11,6 +11,7 @@
 
 
 #include<sys/types.h>
+#include<alloca.h>
 
 
 enum{
@@ -74,6 +75,20 @@ extern void          srand(unsigned);
 extern u_int32_t     arc4random(void);
 extern long long     strtoq (const char *, char **, int);
 extern unsigned long long strtouq (const char *, char **, int);
+extern char			*cgetcap (char *, char *, int);
+extern int			 cgetclose (void);
+extern int			 cgetent (char **, char **, char *);
+extern int			 cgetfirst (char **, char **);
+extern int			 cgetmatch (char *, char *);
+extern int			 cgetnext (char **, char **);
+extern int			 cgetnum (char *, char *, long *);
+extern int			 cgetset (char *);
+extern int			 cgetstr (char *, char *, char **);
+extern int			 cgetustr (char *, char *, char **);
+extern int			 putenv(const char *);
+extern char			*getbsize(int *, long *);
+extern char			*group_from_gid(unsigned long, int);
+extern char			*user_from_uid(unsigned long, int);
 
 
 /* ̤ */
