@@ -10,6 +10,9 @@
 #define sys_system_h
 
 
+#include <sys/cdefs.h>
+
+
 #ifndef ASM_FILE
 
 
@@ -20,18 +23,15 @@ typedef struct{
 
 
 #ifndef KERNEL
+
 enum{
 	/* åȥޥ¸ */
 	USER_ESP_BEG=0xfec00000,
 	PAGE_SIZE=0x1000,
 };
 
-
 extern SYS_INFO *sys_info;
-extern inline int getpagesize()
-{
-	return PAGE_SIZE;
-}
+
 #endif
 
 
