@@ -14,12 +14,14 @@ extern int     tcdrain(int);
 extern int     tcflow(int, int);
 extern int     tcflush(int, int);
 extern int     tcgetattr(int, struct termios*);
-extern int     tcsetattr(int, int, struct termios*);
+extern int     tcsetattr(int, int, const struct termios*);
 extern int     tcsendbreak(int, int);
 extern speed_t cfgetispeed(const struct termios*);
 extern speed_t cfgetospeed(const struct termios*);
 extern int     cfsetispeed(struct termios*, speed_t);
 extern int     cfsetospeed(struct termios*, speed_t);
+extern void    cfmakeraw(struct termios *t);
+extern pid_t   tcgetsid(int);
 
 
 #endif
