@@ -15,7 +15,7 @@ static char empty[1];
 char *__progname = empty;
 
 
-extern int main(int argc,char **argv);
+extern int main(int argc,char **argv,char **envp);
 
 
 void _start(int argc,char *argv[],char **envp)
@@ -31,5 +31,5 @@ void _start(int argc,char *argv[],char **envp)
 		else ++__progname;
 	}
 
-	exit(main(argc,argv));
+	exit(main(argc,argv,envp));
 }
