@@ -25,8 +25,10 @@ struct dirent *readdir(DIR *dirp)
 	if (rest == -1)
 		return NULL;
 
-	if(*dirent.d_name=='\0')return NULL;
-	dirent.d_namlen=strlen(dirent.d_name);
+	if(*dirent.d_name == '\0')
+		return NULL;
+	dirent.d_namlen = strlen(dirent.d_name);
+	dirent.d_fileno = (int)dirp;
 
 	return &dirent;
 }
