@@ -21,12 +21,13 @@ void *sbrk(ptrdiff_t increment)
 	void *begin;
 
 
-	begin=(void*)(sys_info->lastAddr+1);
+	begin = (void*)(sys_info->lastAddr + 1);
 
-	if(increment>0)
+	if (increment > 0)
 	{
-		if(brk((void*)(sys_info->lastAddr+increment))!=0)return NULL;
-		sys_info->lastAddr+=increment;
+		if (brk((void*)(sys_info->lastAddr + increment)) != 0)
+			return NULL;
+		sys_info->lastAddr += increment;
 	}
 
 	return begin;
