@@ -36,7 +36,7 @@ int main(int a,char **b)
 		printf("adr=%d\r",*addr);
 	}
 
-	printf("\nend test_brk\n");
+	printf("end test_brk\n");
 
 	return 0;
 }
