@@ -32,19 +32,9 @@ typedef struct WAIT_QUEUE{
 	int    gate;			/* Lock gate. */
 }WAIT_QUEUE;
 
-/* ޡԤѹ¤ */
-typedef struct WAIT_INTR{
-	PROC *proc;					/* Ԥץ */
-	int flag;					/* ॢȥե饰 */
-}WAIT_INTR;
 
-
-extern void waitProc(PROC_LINK*);
-extern void wakeProc(PROC_LINK*);
 extern void waitSemaphore(WAIT_QUEUE*);
 extern void wakeSemaphore(WAIT_QUEUE*);
-extern int wait_intr(WAIT_INTR*,uint,int);
-extern void wake_intr(WAIT_INTR*,int);
 
 
 #endif
